/*  channel.c --
   
   Part of: bztcl
   Contents: channel interface for compressed files
   Date: Thu Feb 19, 2004
   
   Abstract
   
   
   
   Copyright (c) 2004 Marco Maggi
   
   The author  hereby grant permission to use,  copy, modify, distribute,
   and  license this  software  and its  documentation  for any  purpose,
   provided that  existing copyright notices  are retained in  all copies
   and that  this notice  is included verbatim  in any  distributions. No
   written agreement, license, or royalty  fee is required for any of the
   authorized uses.  Modifications to this software may be copyrighted by
   their authors and need not  follow the licensing terms described here,
   provided that the new terms are clearly indicated on the first page of
   each file where they apply.
   
   IN NO  EVENT SHALL THE AUTHOR  OR DISTRIBUTORS BE LIABLE  TO ANY PARTY
   FOR  DIRECT, INDIRECT, SPECIAL,  INCIDENTAL, OR  CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE  USE OF THIS  SOFTWARE, ITS DOCUMENTATION,  OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR  HAVE BEEN  ADVISED OF  THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR  AND DISTRIBUTORS  SPECIFICALLY  DISCLAIM ANY  WARRANTIES,
   INCLUDING,   BUT   NOT  LIMITED   TO,   THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A    PARTICULAR   PURPOSE,    AND
   NON-INFRINGEMENT.  THIS  SOFTWARE IS PROVIDED  ON AN "AS  IS" BASIS,
   AND  THE  AUTHOR  AND  DISTRIBUTORS  HAVE  NO  OBLIGATION  TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
   
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#if 0
#  define TCLMORE_DEBUG_MESSAGES
#  define TCLMORE_DEBUGGING		1
#endif

#include "bztclInt.h"
#include <errno.h>


/** ------------------------------------------------------------
 ** Preprocessor symbols.
 ** ----------------------------------------------------------*/

#define debug			More_Debug
#define question		More_Question

#define CHANNEL_NAME_LENGTH	(16 + TCL_INTEGER_SPACE)
#define CHANNEL_PATTERN		"bztcl%u"


/** ------------------------------------------------------------
 ** Module's functions prototypes.
 ** ----------------------------------------------------------*/

static Tcl_DriverCloseProc	BzchanClose;
static Tcl_DriverInputProc	BzchanInput;
static Tcl_DriverOutputProc	BzchanOutput;
static Tcl_DriverWatchProc	BzchanWatch;
static Tcl_DriverGetHandleProc	BzchanGetHandle;
static Tcl_DriverSetOptionProc	BzchanSetOption;
static Tcl_DriverGetOptionProc	BzchanGetOption;

static Tcl_TimerProc		BzchanNotifyReadable;
static Tcl_TimerProc		BzchanNotifyWritable;

static More_ChannelDriverSetOptionProc SetOptionWorkFactor;
static More_ChannelDriverSetOptionProc SetOptionBlockSize;
static More_ChannelDriverSetOptionProc SetOptionSmall;

static More_ChannelDriverGetOptionProc GetOptionWorkFactor;
static More_ChannelDriverGetOptionProc GetOptionBlockSize;
static More_ChannelDriverGetOptionProc GetOptionSmall;


/** ------------------------------------------------------------
 ** Module's type declarations.
 ** ----------------------------------------------------------*/

typedef struct Channel {
  Bztcl_File		token;
  Tcl_Channel		channel;
  Tcl_TimerToken	readTimer;
  Tcl_TimerToken	writeTimer;
} Channel;



/** ------------------------------------------------------------
 ** Module's variables.
 ** ----------------------------------------------------------*/

static Tcl_ChannelType Driver = {
  "bztcl_channel",
  TCL_CHANNEL_VERSION_3,
  BzchanClose,
  BzchanInput,
  BzchanOutput,
  NULL, /* Seek */
  BzchanSetOption,
  BzchanGetOption,
  BzchanWatch,
  BzchanGetHandle,
  NULL, /* Close2 */
  NULL, /* BlockMode */
  NULL, /* Flush */
  NULL, /* Handler */
  NULL /* WideSeek */
};

static Tcl_Mutex	counterMutex = NULL;
static int		channelCounter = 0;

static CONST More_ChannelDriverOption optionTable[] = {
  { "-workFactor", SetOptionWorkFactor, GetOptionWorkFactor },
  { "-blockSize",  SetOptionBlockSize,  GetOptionBlockSize },
  { "-small",      SetOptionSmall,      GetOptionSmall },
  { NULL, NULL, NULL }
};


/* Bztcl_MakeChannel --

	Creates  a new  channel  interface to  an  already created  file
	descriptor.

	  BZip2 separates the interface  for readable files from the one
	to  writable  files. This  function  handles this  automatically
	inspecting the state of the descriptor.

   Arguments:

	token -		The file descriptor token.

   Results:

        Returns the channel token.

*/

Tcl_Channel
Bztcl_MakeChannel (Bztcl_File token)
{
  Channel *	data;
  char		channelName[CHANNEL_NAME_LENGTH];
  int		openMode;


  Tcl_MutexLock(&counterMutex);
  sprintf(channelName, CHANNEL_PATTERN, ++channelCounter);
  Tcl_MutexUnlock(&counterMutex);

  openMode = (Bztcl_ReadableFile(token))? TCL_READABLE : TCL_WRITABLE;

  data = (Channel *) ckalloc(sizeof(Channel));
  memset(data, '\0', sizeof(Channel));
  data->token   = token;
  data->channel = Tcl_CreateChannel(&Driver, channelName, data, openMode);

  debug("created channel: name \"%s\", pointer \"%p\"",
	 Tcl_GetChannelName(data->channel), data->channel);
  return data->channel;
}

/* BzchanClose --

	Invoked  by  the  generic   layer  to  clean-up  driver  related
	informations when the channel is closed. 

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.

   Results:

	Must return zero  if the operation is successful,  or a non-zero
	POSIX error code if an error occurs.

*/

int
BzchanClose (ClientData D, Tcl_Interp *interp)
{
  Channel *	data = (Channel *) D;
  More_Error	error;
  int		errorCode = 0;


  debug("closing \"%s\"", Tcl_GetChannelName(data->channel));
  error = Bztcl_Close(data->token);
  if (error)
    {
      errorCode = More_ErrorGetInt(error);
      More_ErrorResult(interp, error);
    }
  else
    {
      ckfree((char *) data);
    }
  return errorCode;
}

/* BzchanInput --

	Invoked by  the generic layer to  read data from  the device and
	store it in an internal buffer.

   Arguments:

	D -		Pointer to the instance structure.
	bufPtr -	Pointer to the output buffer.
	bufLen -	Number of bytes to be read.
	errorCodePtr -	Pointer to a variable that must be filled with
			a POSIX error code in case of error.

   Results:

	Must  return a  non-negative integer  indicating how  many bytes
        were read. In case of error must return -1.

	  If the file was opened  for writing an error is returned, with
	POSIX code EACCES.

*/

int
BzchanInput (ClientData D, char *bufPtr, int bufLen, int *errorCodePtr)
{
  Channel *	data = (Channel *) D;
  More_Block	block;
  More_Error	error;
 
  
  if (! Bztcl_ReadableFile(data->token))
    {
      *errorCodePtr = EACCES;
      return -1;
    }

  debug("request to read %d bytes from \"%s\"", bufLen,
	Tcl_GetChannelName(data->channel));

  if (Bztcl_Eof(data->token))
    {
      *errorCodePtr = 0;
      return 0;
    }

  block.ptr = bufPtr;
  block.len = bufLen;
    
  error = Bztcl_Read(data->token, &block);
  if (! error)
    {
      debug("success, read %d bytes", block.len);
      *errorCodePtr = 0;
      return block.len;
    }
  else
    {
      *errorCodePtr = More_ErrorGetInt(error);
      More_ErrorForget(error);
      return -1;
    }
}

/* BzchanOutput --

	Invoked by the  generic layer to transfer data  from an internal
	buffer to the output device.

   Arguments:

	D -		Pointer to the instance structure.
	bufPtr -	Pointer to the input buffer.
	bufLen -	Number of bytes to be written.
	errorCodePtr -	Pointer to a variable that must be filled with
			a POSIX error code in case of error.

   Results:

	Must  return a  non-negative integer  indicating how  many bytes
        were written; in case of error  must return -1.

	  If the file was opened  for reading an error is returned, with
	POSIX code EACCES.

	  BZip2 interface is such that  if the function returns no error
	all the bytes have been written.

*/

int
BzchanOutput (ClientData D, CONST char *bufPtr, int bufLen, int *errorCodePtr)
{
  Channel *	data = (Channel *) D;
  More_Block	block;
  More_Error	error;
 
  
  if (! Bztcl_WritableFile(data->token))
    {
      *errorCodePtr = EACCES;
      return -1;
    }

  debug("request to write %d bytes to \"%s\"", bufLen,
	Tcl_GetChannelName(data->channel));

  block.ptr = (More_Byte *) bufPtr;
  block.len = bufLen;
    
  error = Bztcl_Write(data->token, block);
  if (! error)
    {
      debug("success");
      *errorCodePtr = 0;
      return block.len;
    }
  else
    {
      *errorCodePtr = More_ErrorGetInt(error);
      More_ErrorForget(error);
      return -1;
    }
}

/* BzchanWatch --

	Invoked by the generic layer to initialise the event notification
	mechanism.

   Arguments:

	D -		Pointer to the instance data.
	eventMask -	An OR-ed combination of TCL_READABLE, TCL_WRITABLE
			and TCL_EXCEPTION. It indicates events the caller
			is interested in.

   Results:

	A file opened  for reading is always readable  and a file opened
	for  writing  is always  writable,  so  if  the requested  event
	complies with the open mode of the file the channel is notified.

*/

void
BzchanWatch (ClientData D, int eventMask)
{
  Channel *	data = (Channel *) D;


  debug("enabling events for %s: readable %s, writable %s",
	Tcl_GetChannelName(data->channel),
	question(eventMask & TCL_READABLE),
	question(eventMask & TCL_WRITABLE));

  eventMask &= (TCL_READABLE & TCL_WRITABLE);

  if (Bztcl_ReadableFile(data->token))
    {
      if ((eventMask & TCL_READABLE) && (! data->readTimer))
	{
	  data->readTimer = \
	    Tcl_CreateTimerHandler(1, BzchanNotifyReadable, (ClientData) data);
	}
      else if ((! (eventMask & TCL_READABLE)) && (data->readTimer))
	{
	  Tcl_DeleteTimerHandler(data->readTimer);
	  data->readTimer = NULL;
	}
    }
  if (Bztcl_WritableFile(data->token))
    {
      if ((eventMask & TCL_WRITABLE) && (! data->writeTimer))
	{
	  data->writeTimer = \
	    Tcl_CreateTimerHandler(1, BzchanNotifyWritable, (ClientData) data);
	}
      else if ((! (eventMask & TCL_WRITABLE)) && (data->writeTimer))
	{
	  Tcl_DeleteTimerHandler(data->writeTimer);
	  data->writeTimer = NULL;
	}
    }
}

/* BzchanNotifyReadable --

	Invoked by the event loop  to notify the generic layer about the
	readability of the channel.

   Arguments:

	D -		File token.

   Results:

	The generic layer is notifier of the event.

*/

void
BzchanNotifyReadable (ClientData D)
{
  Channel *	data = (Channel *) D;

  Tcl_NotifyChannel(data->channel, TCL_READABLE);
  data->readTimer = NULL;
}

/* BzchanNotifyWritable --

	Invoked by the event loop  to notify the generic layer about the
	writability of the channel.

   Arguments:

	D -		File token.

   Results:

	The generic layer is notifier of the event.

*/

void
BzchanNotifyWritable (ClientData D)
{
  Channel *	data = (Channel *) D;

  Tcl_NotifyChannel(data->channel, TCL_WRITABLE);
  data->writeTimer = NULL;
}

/* BzchanGetHandle --

	Invoked by the generic layer to retrieve the device-specific
	handler for the channel.

   Arguments:

	D -		Pointer to the instance data.
	direction -	The channel direction: TCL_READABLE or
			TCL_WRITABLE.
	handlePtr -	Pointer to the variable that will hold
			the handler.

   Results:

	Sends  to  the  generic  layer  the  identifier  of  the  stream
	channel. Returns TCL_OK.

*/

int
BzchanGetHandle (ClientData D, int direction, ClientData *handlePtr)
{
  Channel *	data = (Channel *) D;

  *handlePtr = (ClientData) Bztcl_GetHandle(data->token);
  return TCL_OK;
}

/* BzchanSetOption --

	Invoked  by the  generic layer  to set  the value  of  a channel
	specific option.

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.
	optionName -	Pointer to a string representing the option
			name.
	optionValue -	Pointer to a string representing the new
			value.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
BzchanSetOption (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     CONST char *	optionValue;
{
  Channel *	data = (Channel *) D;

  return More_ChannelDriverSetOption(optionTable, data->channel,
				     D, interp, optionName, optionValue);
}

/* SetOptionWorkFactor --

	Configures the work factor of the file descriptor.

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.
	optionName -	Pointer to a string representing the option
			name.
	optionValue -	Pointer to a string representing the new
			value.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
SetOptionWorkFactor (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     CONST char *	optionValue;
{
  Channel *	data = (Channel *) D;
  Tcl_Obj *	objPtr;
  int		workFactor, e;

  
  if (! Bztcl_WritableFile(data->token))
    {
      return More_LogicErrorStr(interp, "file not writable");
    }

  objPtr = Tcl_NewStringObj(optionValue, -1);
  Tcl_IncrRefCount(objPtr);
  e = Bztcl_GetWorkFactorFromObj(interp, objPtr, &workFactor);
  Tcl_DecrRefCount(objPtr);
  if (TCL_OK == e)
    {
      Bztcl_SetWorkFactor(data->token, workFactor);
      return TCL_OK;
    }
  else
    {
      return e;
    }
}

/* SetOptionBlockSize --

	Configures the block size of the file descriptor.

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.
	optionName -	Pointer to a string representing the option
			name.
	optionValue -	Pointer to a string representing the new
			value.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
SetOptionBlockSize (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     CONST char *	optionValue;
{
  Channel *	data = (Channel *) D;
  Tcl_Obj *	objPtr;
  int		blockSize, e;

  
  if (! Bztcl_WritableFile(data->token))
    {
      return More_LogicErrorStr(interp, "file not writable");
    }

  objPtr = Tcl_NewStringObj(optionValue, -1);
  Tcl_IncrRefCount(objPtr);
  e = Bztcl_GetBlockSizeFromObj(interp, objPtr, &blockSize);
  Tcl_DecrRefCount(objPtr);
  if (TCL_OK == e)
    {
      Bztcl_SetBlockSize(data->token, blockSize);
      return TCL_OK;
    }
  else
    {
      return e;
    }
}

/* SetOptionSmall --

	Configures the small memory mode of the file descriptor.

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.
	optionName -	Pointer to a string representing the option
			name.
	optionValue -	Pointer to a string representing the new
			value.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
SetOptionSmall (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     CONST char *	optionValue;
{
  Channel *	data = (Channel *) D;
  Tcl_Obj *	objPtr;
  int		small, e;

  
  if (! Bztcl_ReadableFile(data->token))
    {
      return More_LogicErrorStr(interp, "file not readable");
    }

  objPtr = Tcl_NewStringObj(optionValue, -1);
  Tcl_IncrRefCount(objPtr);
  e = Tcl_GetBooleanFromObj(interp, objPtr, &small);
  Tcl_DecrRefCount(objPtr);
  if (TCL_OK == e)
    {
      Bztcl_SetSmall(data->token, small);
      return TCL_OK;
    }
  else
    {
      return e;
    }
}

/* BzchanGetOption --

	Invoked by the generic layer  to retrieve the value of a channel
	specific option.

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.
	optionName -	The requested option name, NULL if all the
			options are requested.
	optionValue -	Pointer to the output string.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
BzchanGetOption (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     Tcl_DString *	optionValue;
{
  Channel *	data = (Channel *) D;

  return More_ChannelDriverGetOption(optionTable, data->channel,
				     D, interp, optionName, optionValue);
}

/* GetOptionWorkFactor --

	Extracts the work factor from the file descriptor.

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.
	optionName -	The requested option name, NULL if all the
			options are requested.
	optionValue -	Pointer to the output string.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
GetOptionWorkFactor (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     Tcl_DString *	optionValue;
{
  Channel *	data = (Channel *) D;

  More_DStringPrintf(optionValue, "%d", Bztcl_GetWorkFactor(data->token));
  return TCL_OK;
}

/* GetOptionBlockSize --

	Extracts the block size from the file descriptor.

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.
	optionName -	The requested option name, NULL if all the
			options are requested.
	optionValue -	Pointer to the output string.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
GetOptionBlockSize (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     Tcl_DString *	optionValue;
{
  Channel *	data = (Channel *) D;

  More_DStringPrintf(optionValue, "%d", Bztcl_GetBlockSize(data->token));
  return TCL_OK;
}

/* GetOptionSmall --

	Extracts the small mode from the file descriptor.

   Arguments:

	D -		Pointer to the instance structure.
	interp -	If non-NULL: the interpreter used to report
			errors.
	optionName -	The requested option name, NULL if all the
			options are requested.
	optionValue -	Pointer to the output string.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

int
GetOptionSmall (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     Tcl_DString *	optionValue;
{
  Channel *	data = (Channel *) D;

  More_DStringPrintf(optionValue, "%d", Bztcl_GetSmall(data->token));
  return TCL_OK;
}


/* end of file */
/*
Local Variables:
mode: c
page-delimiter: "^$"
End:
*/
