/* commands.c --
   
   Part of: BZTCL
   Contents: command functions
   Date: Fri May  3, 2002
   
   Abstract
   
	This  file  provides  the   functions  that  implement  the  TCL
	commands.
   
   Copyright (c) 2002, 2003, 2004 Marco Maggi
   
   The author hereby grant  permission to use, copy, modify, distribute,
   and  license this  software and  its documentation  for  any purpose,
   provided that  existing copyright notices are retained  in all copies
   and that  this notice is  included verbatim in any  distributions. No
   written agreement, license, or royalty fee is required for any of the
   authorized uses.   Modifications to this software  may be copyrighted
   by their  authors and need  not follow the licensing  terms described
   here, provided that the new  terms are clearly indicated on the first
   page of each file where they apply.
   
   IN NO EVENT  SHALL THE AUTHOR OR DISTRIBUTORS BE  LIABLE TO ANY PARTY
   FOR DIRECT,  INDIRECT, SPECIAL, INCIDENTAL,  OR CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE USE OF  THIS SOFTWARE, ITS  DOCUMENTATION, OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR HAVE  BEEN ADVISED  OF THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR AND  DISTRIBUTORS SPECIFICALLY  DISCLAIM  ANY WARRANTIES,
   INCLUDING,   BUT  NOT   LIMITED   TO,  THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A   PARTICULAR    PURPOSE,   AND
   NON-INFRINGEMENT.  THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, AND
   THE   AUTHOR  AND   DISTRIBUTORS  HAVE   NO  OBLIGATION   TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#include <string.h>
#include <stdio.h>

#if 0
#  define TCLMORE_DEBUG_MESSAGES
#  define TCLMORE_DEBUGGING	1
#endif

#ifdef BZTCL_STUB_TEST
#  include "bztclTest.h"
#else
#  include "bztclInt.h"
#endif


/** ------------------------------------------------------------
 ** Preprocessor symbols.
 ** ----------------------------------------------------------*/

#define debug		More_Debug
#define question	More_Question


/** ------------------------------------------------------------
 ** Extractor prototypes.
 ** ----------------------------------------------------------*/



/** ------------------------------------------------------------
 ** Command tables.
 ** ----------------------------------------------------------*/

#include "declaration.c"

#ifndef BZTCL_STUB_TEST

int
Bztcl_CreateCommands (Tcl_Interp *interp)
{
  More_Error	error;


  error = More_CreateCommands(interp, NULL, Bztcl_CommandsTable);
  return (error)? More_ErrorResult(interp, error) : TCL_OK;
}

#else
#warning using bztcl stub

int
Bztcltest_CreateCommands (Tcl_Interp *interp)
{
  More_Error	error;
  

  error = More_CreateCommands(interp, "test", Bztcl_CommandsTable);
  return (error)? More_ErrorResult(interp, error) : TCL_OK;
}

#endif


/* BzlibCompressFunction --

	Implementation function for the compress command.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns TCL_OK or TCL_ERROR. The return value of the TCL command
        is the compressed data.

*/

int
BzlibCompressFunction (More_CommandFrame frame)
{
  CompressState	*	state = (CompressState *) frame->stateStructPtr;
  Tcl_Obj *		result;
  More_Error		error;


  error = Bztcl_CompressObj(state->dataObj, &(state->config), &result);
  if (error)
    {
      return More_ErrorResult(frame->interp, error);
    }

  Tcl_SetObjResult(frame->interp, result);
  return TCL_OK;
}

/* BzlibDecompressFunction --

	Implementation function for the uncompress command.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns TCL_OK or TCL_ERROR. The return value of the TCL command
        is the uncompressed data.

*/

int
BzlibDecompressFunction (More_CommandFrame frame)
{
  DecompressState *	state = (DecompressState *) frame->stateStructPtr;
  Tcl_Obj *		result;
  More_Error		error;


  error = Bztcl_UncompressObj(state->dataObj, &(state->config),
			      state->size, &result);
  if (error)
    {
      return More_ErrorResult(frame->interp, error);
    }

  Tcl_SetObjResult(frame->interp, result);
  return TCL_OK;
}

/* BzlibOpenFunction --

	Implementation function for the open command.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns TCL_OK or TCL_ERROR. The return value of the TCL command
        is the channel identifier.

*/

int
BzlibOpenFunction (More_CommandFrame frame)
{
  OpenState *	state = (OpenState *) frame->stateStructPtr;
  Bztcl_File	descriptor;
  Tcl_Channel	channel;
  More_Error	error;


  debug("request to open \"%s\", readable %s, writable %s",
	Tcl_GetString(state->fileName),
	question(state->openMode & TCL_READABLE),
	question(state->openMode & TCL_WRITABLE));

  if ((state->openMode & TCL_READABLE) && (state->openMode & TCL_WRITABLE))
    {
      Tcl_SetResult(frame->interp, "wrong open mode", TCL_STATIC);
      return More_LogicError(frame->interp);
    }

  error = Bztcl_Open(Tcl_GetString(state->fileName),
		     state->openMode, &descriptor);
  if (error)
    {
      return More_ErrorResult(frame->interp, error);
    }

  channel = Bztcl_MakeChannel(descriptor);

  Tcl_RegisterChannel(frame->interp, channel);
  Tcl_SetObjResult(frame->interp,
		   Tcl_NewStringObj(Tcl_GetChannelName(channel), -1));
  return TCL_OK;
}

/* BzlibStreamFunction --

	Stacks a transformation above an existing channel.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns TCL_OK or TCL_ERROR. The return value of the TCL command
        is the channel name.

*/

int
BzlibStreamFunction (More_CommandFrame frame)
{
  StackState *	state = (StackState *) frame->stateStructPtr;
  Bztcl_Stream	input = NULL;
  Bztcl_Stream	output = NULL;
  Tcl_Channel	channel;
  More_Error	error;

  
  debug("stacking stream, readable %s, writable %s, blockSize %d, workFactor %d, small %d",
	question(state->openMode & TCL_READABLE),
	question(state->openMode & TCL_WRITABLE),
	state->config.blockSize, state->config.workFactor, state->config.small);

  if (state->openMode & TCL_READABLE)
    {
      debug("input stream compress? %s", question(state->compressInput));
      error = Bztcl_StreamInit(&input, state->compressInput, &(state->config));
      if (error)
	{
	  return More_ErrorResult(frame->interp, error);
	}
    }
  
  if (state->openMode & TCL_WRITABLE)
    {
      debug("output stream compress? %s", question(state->compressOutput));
      error = Bztcl_StreamInit(&output, state->compressOutput, &(state->config));
      if (error)
	{
	  return More_ErrorResult(frame->interp, error);
	}
    }
  
  channel = Bztcl_MakeTransform(input, output, state->subChannel);
  Tcl_SetObjResult(frame->interp,
		   Tcl_NewStringObj(Tcl_GetChannelName(channel), -1));
  return TCL_OK;
}

/* BzlibInfoIsworkfactorFunction --

	Implementation  function  for   the  [bzlib  info  isWorkFactor]
	command.

   Arguments:

	frame -		Invocation parameters.

   Results:

	Returns TCL_OK. The  return value of the TCL  command is true if
	the command argument is a valid work factor value.

*/

int
BzlibInfoIsworkfactorFunction (More_CommandFrame frame)
{
  ObjectState *	state = (ObjectState *) frame->stateStructPtr;
  int		value, e;

  e = Bztcl_GetWorkFactorFromObj(frame->interp, state->object, &value);
  Tcl_SetObjResult(frame->interp,
		   Tcl_NewBooleanObj((TCL_OK == e)? 1 : 0));
  return TCL_OK;
}

/* BzlibInfoIsblocksizeFunction --

	Implementation  function   for  the  [bzlib   info  isBlockSize]
	command.

   Arguments:

	frame -		Invocation parameters.

   Results:

	Returns TCL_OK. The  return value of the TCL  command is true if
	the command argument is a valid block size value.

*/

int
BzlibInfoIsblocksizeFunction (More_CommandFrame frame)
{
  ObjectState *	state = (ObjectState *) frame->stateStructPtr;
  int		value, e;

  e = Bztcl_GetBlockSizeFromObj(frame->interp, state->object, &value);
  Tcl_SetObjResult(frame->interp,
		   Tcl_NewBooleanObj((TCL_OK == e)? 1 : 0));
  return TCL_OK;
}

/* BzlibInfoVersionFunction --

	Implementation function for the [bzlib info version] command.

   Arguments:

	frame -		Invocation parameters.

   Results:

        Returns TCL_OK. The return value of the TCL command is the BZip2
        version string.

*/

int
BzlibInfoVersionFunction (More_CommandFrame frame)
{
  Tcl_SetObjResult(frame->interp, Tcl_NewStringObj(Bztcl_Version(), -1));
  return TCL_OK;
}

/** ------------------------------------------------------------
 ** Value extractors.
 ** ----------------------------------------------------------*/

int
GetObjFromArg (More_ExtractorFrame frame)
{
  return More_GetObjFromArg(frame);
}

int
GetIntFromArg (More_ExtractorFrame frame)
{
  return More_GetIntFromArg(frame);
}

int
GetChannelFromArg (More_ExtractorFrame frame)
{
  StackState *	state = (StackState *) frame->commandFrame->stateStructPtr;

  return More_GetChannelFromObj(frame->commandFrame->interp, frame->srcObj,
				&(state->subChannel),
				&(state->subOpenMode));
}

int
GetOpenModeFromArg (More_ExtractorFrame frame)
{
  return  More_GetOpenModeFromObj(frame->commandFrame->interp,
				  frame->srcObj,
				  TCLMORE_READ_OR_WRITE,
				  (int *) frame->fieldPtr);
}

int
GetTransformModeFromArg (More_ExtractorFrame frame)
{
  StackState *	state = (StackState *) frame->commandFrame->stateStructPtr;

  return More_GetTransformOpenModeFromObj(frame->commandFrame->interp,
					  frame->srcObj,
					  state->subOpenMode,
					  0, (int *) frame->fieldPtr);
}

int
GetStreamActionFromArg (More_ExtractorFrame frame)
{
  static CONST char *	actions[] = { "compress", "decompress", NULL };
  int			e, index;
  int *			actionPtr;


  e = Tcl_GetIndexFromObj(frame->commandFrame->interp,
			  frame->srcObj,
			  actions, "action", TCL_EXACT, &index);
  if (TCL_OK != e)
    {
      return e;
    }

  actionPtr  = (int *) frame->fieldPtr;
  *actionPtr = (! index);
  return TCL_OK;
}


int
GetWorkFactorFromArg (More_ExtractorFrame frame)
{
  return Bztcl_GetWorkFactorFromObj(frame->commandFrame->interp,
				    frame->srcObj,
				    (int *) frame->fieldPtr);
}

int
GetBlockSizeFromArg (More_ExtractorFrame frame)
{
  return Bztcl_GetBlockSizeFromObj(frame->commandFrame->interp,
				   frame->srcObj,
				   (int *) frame->fieldPtr);
}

int
GetSmallFromArg (More_ExtractorFrame frame)
{
  int *		smallPtr;


  smallPtr = (int *) frame->fieldPtr;
  *smallPtr = 1;
  return TCL_OK;
}

/* end of file */
/*
  Local Variables:
  page-delimiter: "^"
  End:
*/
