/*
  This file is meant to be included (with #include) in a source
  file provided by you that implements the command functions. Your
  file must include the TCL header ("tcl.h") first and define the
  functions later.

    The reason of this inclusion is that here are declared the struct
  typedefs of the command invocation states. These types must be
  visible by the command functions. The stuff declared in this file
  uses the "static" keyword as much as possible.

    The command functions should be small: they should do the job by
  invoking other functions, the ones that the extension should export
  in its stub table.
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#include "tclmore.h"



/** ------------------------------------------------------------
 ** Type declarations.
 ** ----------------------------------------------------------*/

/* CompressState --

  This structure is used to hold the invocation state of the
  following commands:

                    [bzlib compress   data ?options?]
*/
typedef struct CompressState {
  Tcl_Obj *               dataObj;
  Bztcl_Config            config;
} CompressState;

static CONST CompressState CompressStateDefaults = {
  NULL, BZTCL_DEFAULT_CONFIG
};

/* DecompressState --

  This structure is used to hold the invocation state of the
  following commands:

                    [bzlib decompress   data ?options?]
*/
typedef struct DecompressState {
  Tcl_Obj *               dataObj;
  int                     size;
  Bztcl_Config            config;
} DecompressState;

static CONST DecompressState DecompressStateDefaults = {
  NULL, 0, BZTCL_DEFAULT_CONFIG
};

/* OpenState --

  This structure is used to hold the invocation state of the
  following commands:

                    [bzlib open   fileName mode ?options?]
*/
typedef struct OpenState {
  Tcl_Obj *               fileName;
  int                     openMode;
  Bztcl_Config            config;
} OpenState;

static CONST OpenState OpenStateDefaults = {
  NULL, 0, BZTCL_DEFAULT_CONFIG
};

/* ObjectState --

  This structure is used to hold the invocation state of the
  following commands:


*/
typedef struct ObjectState {
  Tcl_Obj *               object;
} ObjectState;

static CONST ObjectState ObjectStateDefaults = {
  NULL
};

/* StackState --

  This structure is used to hold the invocation state of the
  following commands:

                    [bzlib stream   channel mode ?options?]
*/
typedef struct StackState {
  Tcl_Channel             subChannel;
  int                     subOpenMode;
  int                     openMode;
  int                     compressInput;
  int                     compressOutput;
  Bztcl_Config            config;
} StackState;

static CONST StackState StackStateDefaults = {
  NULL, 0, 0, 1, 0, BZTCL_DEFAULT_CONFIG
};

/* IntegerState --

  This structure is used to hold the invocation state of the
  following commands:

                    [bzlib info isWorkFactor   workFactor]
                    [bzlib info isBlockSize   blockSize]
*/
typedef struct IntegerState {
  int                     integer;
} IntegerState;

static CONST IntegerState IntegerStateDefaults = {
  0
};



/** ------------------------------------------------------------
 ** Implementation functions prototypes.
 ** ----------------------------------------------------------*/

/*
  You must provide the implementation of each of these functions in
  this file. When the function is invoked, the state invocation
  structure has been already filled with values from the command line
  arguments and options, with no errors.

    The function should look like the following:

  int
  MyCommandFunction (More_CommandFrame frame)
  {
     MyStateStruct *  state = (MyStateStruct *) frame->stateStructPtr;
     Tcl_Obj *        result;

     *
       Make use of the fields of "*state" to build a result by
       invoking functions from your library.
     *
     result = Mylib_DoSomething(state->value, state->another_value);

     Tcl_SetObjResult(frame->interp, result);
     return TCL_OK;
  }

    Being the implementation of a TCL command, this function is
  declared "static": it is useless to share this function among
  different modules. Remember that the role of a "More_CmdFunc"
  is to be an interface between the state invocation structure and
  the functions of your library that do the "real work".
*/

static More_CmdFunc         BzlibCompressFunction;
static More_CmdFunc         BzlibDecompressFunction;
static More_CmdFunc         BzlibOpenFunction;
static More_CmdFunc         BzlibStreamFunction;
static More_CmdFunc          BzlibInfoIsworkfactorFunction;
static More_CmdFunc          BzlibInfoIsblocksizeFunction;
static More_CmdFunc          BzlibInfoVersionFunction;


/** ------------------------------------------------------------
 ** Extractor functions prototypes.
 ** ----------------------------------------------------------*/

/*
  You must provide the implementation of each of the static extractors
  in this file. Functions declared EXTERN can be implemented in other
  modules of your extension or in other libraries that are dependencies
  of this one (for example the TCLMORE library implements some common
  extractors).

    The functions must be available as pointers: you cannot directly use
  functions exported in a stub table; this is because the pointer must
  be included in a statically allocated data structure. Remember that
  stub function names are available as macros, each macro references
  an element in a vector of function pointers in another library: the
  value of the function pointer is available only at execution time.

    The only way to use an extractor from a stub table is to declare
  another extractor (with a similar name) that wraps the real function.
  Example:

     More_Extractor MyExtractor;
     ...
     int
     MyExtractor (More_ExtractorFrame frame)
     {
       return RealExtractor(frame);
     }

  "RealExtractor" can be the macro of a stub function.

    If a prototype is omitted: it is your responsibility to declare it
  before the line that includes this file in your source module. Obviously
  the declaration can be in an external header file; this is particularly
  useful if your extension is using a wrapper extractor for a real extractor
  from another extension, and the extractor is used in more than one
  set of commands.

    A common extractor has the following structure, lets assume that
  "Type" is the type of value we want:

     EXTERN int Mylib_GetTypeFromObj _ANSI_ARGS_((Tcl_Interp * interp,
                                                  Tcl_Obj *    srcObj,
                                                  Type *       valueVar));
     ...
     int
     TypeExtractor (More_ExtractorFrame frame)
     {
        return Mylib_GetTypeFromObj(frame->commandFrame->interp,
                                    frame->srcObj,
                                    (Type *) frame->fieldPtr);       
     }

  or:

     int
     TypeExtractor (More_ExtractorFrame frame)
     {
        int   e;

        e = Mylib_GetTypeFromObj(frame->commandFrame->interp,
                                 frame->srcObj,
                                 (Type *) frame->fieldPtr);       
        return ((TCL_OK == e)? e : More_LogicError(interp));
     }

  the implementation of a "GetTypeFromObj" function to do the real
  work is recommended because such functions can be used by other
  modules, and even exported in the stub table.

    Remember that the field pointer can reference a whole structure
  embedded in the state invocation one.

    At present an extractor can validate arguments only with complete
  independence from the current process state, that is it can test that
  a value is in the correct space and range. Coherence tests between
  argument values and process state must be performed by the command
  implementation functions. This is not exactly true, but it is the
  recommended way to do things.
*/

static More_Extractor      GetObjFromArg;
static More_Extractor      GetBlockSizeFromArg;
static More_Extractor      GetWorkFactorFromArg;
static More_Extractor      GetIntFromArg;
static More_Extractor      GetSmallFromArg;
static More_Extractor      GetOpenModeFromArg;
static More_Extractor      GetChannelFromArg;
static More_Extractor      GetTransformModeFromArg;
static More_Extractor      GetStreamActionFromArg;


/** ------------------------------------------------------------
 ** Command arguments tables.
 ** ----------------------------------------------------------*/

#define bzlib_compress_ARGUMENT_NUMBER	1
static CONST More_CommandArgument BzlibCompressArguments[1] = {
  { GetObjFromArg,       offsetof(struct CompressState, dataObj) }
};

#define bzlib_decompress_ARGUMENT_NUMBER	1
static CONST More_CommandArgument BzlibDecompressArguments[1] = {
  { GetObjFromArg,       offsetof(struct DecompressState, dataObj) }
};

#define bzlib_open_ARGUMENT_NUMBER	2
static CONST More_CommandArgument BzlibOpenArguments[2] = {
  { GetObjFromArg,       offsetof(struct OpenState, fileName) },
  { GetOpenModeFromArg,  offsetof(struct OpenState, openMode) }
};

#define bzlib_stream_ARGUMENT_NUMBER	2
static CONST More_CommandArgument BzlibStreamArguments[2] = {
  { GetChannelFromArg,   offsetof(struct StackState, subChannel) },
  { GetTransformModeFromArg, offsetof(struct StackState, openMode) }
};

#define bzlib_info_isWorkFactor_ARGUMENT_NUMBER	1
static CONST More_CommandArgument BzlibInfoIsworkfactorArguments[1] = {
  { GetObjFromArg,       offsetof(struct IntegerState, integer) }
};

#define bzlib_info_isBlockSize_ARGUMENT_NUMBER	1
static CONST More_CommandArgument BzlibInfoIsblocksizeArguments[1] = {
  { GetObjFromArg,       offsetof(struct IntegerState, integer) }
};

/* No arguments for [bzlib info version]. */
#define bzlib_info_version_ARGUMENT_NUMBER	0



/** ------------------------------------------------------------
 ** Command options tables.
 ** ----------------------------------------------------------*/

static CONST More_CommandOption BzlibCompressOptions[3] = {
  { "-blockSize", 1, GetBlockSizeFromArg, offsetof(struct CompressState, config.blockSize) },
  { "-workFactor", 1, GetWorkFactorFromArg, offsetof(struct CompressState, config.workFactor) },
  { NULL, 0, NULL, 0 }
};

static CONST More_CommandOption BzlibDecompressOptions[3] = {
  { "-size", 1, GetIntFromArg, offsetof(struct DecompressState, size) },
  { "-small", 0, GetSmallFromArg, offsetof(struct DecompressState, config.small) },
  { NULL, 0, NULL, 0 }
};

static CONST More_CommandOption BzlibOpenOptions[4] = {
  { "-blockSize", 1, GetBlockSizeFromArg, offsetof(struct OpenState, config.blockSize) },
  { "-workFactor", 1, GetWorkFactorFromArg, offsetof(struct OpenState, config.workFactor) },
  { "-small", 0, GetSmallFromArg, offsetof(struct OpenState, config.small) },
  { NULL, 0, NULL, 0 }
};

static CONST More_CommandOption BzlibStreamOptions[6] = {
  { "-input", 1, GetStreamActionFromArg, offsetof(struct StackState, compressInput) },
  { "-output", 1, GetStreamActionFromArg, offsetof(struct StackState, compressOutput) },
  { "-blockSize", 1, GetBlockSizeFromArg, offsetof(struct StackState, config.blockSize) },
  { "-workFactor", 1, GetWorkFactorFromArg, offsetof(struct StackState, config.workFactor) },
  { "-small", 0, GetSmallFromArg, offsetof(struct StackState, config.small) },
  { NULL, 0, NULL, 0 }
};

/* No options for [bzlib info isWorkFactor]. */

/* No options for [bzlib info isBlockSize]. */

/* No options for [bzlib info version]. */


/** ------------------------------------------------------------
 ** Leaf commands array.
 ** ----------------------------------------------------------*/

/*
  There is a leaf structure for each "More_CmdFunc". Each structure
  has references to the argument and option tables. It is not your
  business to mess with this structure.
*/

static More_CommandTreeLeaf commandTreeLeaves[] = {
  /* (0) synopsis: bzlib compress data ?options? */
  { 2, BzlibCompressFunction, sizeof(CompressState), (CONST ClientData)&CompressStateDefaults,
    bzlib_compress_ARGUMENT_NUMBER, BzlibCompressArguments, BzlibCompressOptions,
    "data ?options?" },

  /* (1) synopsis: bzlib decompress data ?options? */
  { 2, BzlibDecompressFunction, sizeof(DecompressState), (CONST ClientData)&DecompressStateDefaults,
    bzlib_decompress_ARGUMENT_NUMBER, BzlibDecompressArguments, BzlibDecompressOptions,
    "data ?options?" },

  /* (2) synopsis: bzlib open fileName mode ?options? */
  { 2, BzlibOpenFunction, sizeof(OpenState), (CONST ClientData)&OpenStateDefaults,
    bzlib_open_ARGUMENT_NUMBER, BzlibOpenArguments, BzlibOpenOptions,
    "fileName mode ?options?" },

  /* (3) synopsis: bzlib stream channel mode ?options? */
  { 2, BzlibStreamFunction, sizeof(StackState), (CONST ClientData)&StackStateDefaults,
    bzlib_stream_ARGUMENT_NUMBER, BzlibStreamArguments, BzlibStreamOptions,
    "channel mode ?options?" },

  /* (4) synopsis: bzlib info isWorkFactor workFactor */
  { 3, BzlibInfoIsworkfactorFunction, sizeof(IntegerState), (CONST ClientData)&IntegerStateDefaults,
    bzlib_info_isWorkFactor_ARGUMENT_NUMBER, BzlibInfoIsworkfactorArguments, NULL,
    "workFactor" },

  /* (5) synopsis: bzlib info isBlockSize blockSize */
  { 3, BzlibInfoIsblocksizeFunction, sizeof(IntegerState), (CONST ClientData)&IntegerStateDefaults,
    bzlib_info_isBlockSize_ARGUMENT_NUMBER, BzlibInfoIsblocksizeArguments, NULL,
    "blockSize" },

  /* (6) synopsis: bzlib info version  */
  { 3, BzlibInfoVersionFunction, 0, (CONST ClientData)NULL,
    bzlib_info_version_ARGUMENT_NUMBER, NULL, NULL,
    NULL }
};


/** ------------------------------------------------------------
 ** Node commands array.
 ** ----------------------------------------------------------*/

/*
  These are node structures that declare subcommands with subcommands.
  It is not your business to mess with this structure.
*/

static More_CommandTreeNode BzlibInfoCommandNode[] = {
  { "isWorkFactor", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[4] },
  { "isBlockSize", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[5] },
  { "version", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[6] },
  { NULL, TCLMORE_UNSAFE_COMMAND, NULL, NULL }
};

static More_CommandTreeNode BzlibCommandNode[] = {
  { "compress", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[0] },
  { "decompress", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[1] },
  { "open", TCLMORE_UNSAFE_COMMAND,  NULL, &commandTreeLeaves[2] },
  { "stream", TCLMORE_SAFE_COMMAND,  NULL, &commandTreeLeaves[3] },
  { "info", TCLMORE_SAFE_COMMAND,  BzlibInfoCommandNode, NULL },
  { NULL, TCLMORE_UNSAFE_COMMAND, NULL, NULL }
};



/** ------------------------------------------------------------
 ** Root commands table.
 ** ----------------------------------------------------------*/

/*
  This array of structures declares the root commands. There is one of
  them for each <maincommand> and <command> markups with no ancestors
  in the original XML document. It is not your business to mess with
  the content of this array.

    You may create commands declared in this array with the following
  chunk of code:

    error = More_CreateCommands(interp, "namesp", Bztcl_CommandsTable);
    if (error)
      {
	return More_ErrorResult(interp, error);
      }

  where "namesp" is the name of the namespace in which the command
  must be created. If the namespace is the root one: you can use NULL.
*/

More_CommandTreeNode Bztcl_CommandsTable[] = {
  { "bzlib", TCLMORE_SAFE_COMMAND, BzlibCommandNode, NULL },
  { NULL, 0, NULL, NULL }
};


/* end of file */
/*
Local Variables:
mode: c
page-delimiter: "^$"
End:
*/

