/* interface.c --
   
   Part of: BZTCL
   Contents: interface to BZip2
   Date: Tue May 14, 2002
   
   Abstract
   
      	This file defines the interface  to the functions of BZip2.
   
   Copyright (c) 2002, 2003, 2004 Marco Maggi
   
   The author hereby grant  permission to use, copy, modify, distribute,
   and  license this  software and  its documentation  for  any purpose,
   provided that  existing copyright notices are retained  in all copies
   and that  this notice is  included verbatim in any  distributions. No
   written agreement, license, or royalty fee is required for any of the
   authorized uses.   Modifications to this software  may be copyrighted
   by their  authors and need  not follow the licensing  terms described
   here, provided that the new  terms are clearly indicated on the first
   page of each file where they apply.
   
   IN NO EVENT  SHALL THE AUTHOR OR DISTRIBUTORS BE  LIABLE TO ANY PARTY
   FOR DIRECT,  INDIRECT, SPECIAL, INCIDENTAL,  OR CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE USE OF  THIS SOFTWARE, ITS  DOCUMENTATION, OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR HAVE  BEEN ADVISED  OF THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR AND  DISTRIBUTORS SPECIFICALLY  DISCLAIM  ANY WARRANTIES,
   INCLUDING,   BUT  NOT   LIMITED   TO,  THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A   PARTICULAR    PURPOSE,   AND
   NON-INFRINGEMENT.  THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, AND
   THE   AUTHOR  AND   DISTRIBUTORS  HAVE   NO  OBLIGATION   TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#if 0
#define TCLMORE_DEBUG_MESSAGES
#define TCLMORE_DEBUGGING	1
#endif

#include <assert.h>
#include <limits.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include "bztclInt.h"
#include "bzlib.h"


/** ------------------------------------------------------------
 ** Preprocessor symbols.
 ** ----------------------------------------------------------*/

#define debug		More_Debug
#define question	More_Question

#define DEFAULT_BLOCK_SIZE	3
#define DEFAULT_WORK_FACTOR	30

#define COMPRESSION_INPUT_BUFFER_SIZE		(4096*8)
#define COMPRESSION_OUTPUT_BUFFER_SIZE		(4096*4)

#define DECOMPRESSION_INPUT_BUFFER_SIZE		(4096*4)
#define DECOMPRESSION_OUTPUT_BUFFER_SIZE	(4096*8)


/** ------------------------------------------------------------
 ** Module's type declarations.
 ** ----------------------------------------------------------*/

typedef struct Channel {
  FILE *	file;
  BZFILE *	bzfile;
  int		openMode;
  int		eof;
  Bztcl_Config	config;
} Channel;


/** ------------------------------------------------------------
 ** Module's functions prototypes.
 ** ----------------------------------------------------------*/

static More_Error ReadOpen	_ANSI_ARGS_((Channel *descriptor));
static More_Error WriteOpen	_ANSI_ARGS_((Channel *descriptor));
More_Error Bztcl_MakeErrorDescriptor	_ANSI_ARGS_((int bzerror));

#define MakeErrorDescriptor	Bztcl_MakeErrorDescriptor


/** ------------------------------------------------------------
 ** Module's variables.
 ** ----------------------------------------------------------*/

static Bztcl_Config FileDefaultConfig = BZTCL_DEFAULT_CONFIG;


More_Error
Bztcl_CompressObj (srcObj, cfg, dstObjVar)
     Tcl_Obj *		srcObj;
     Bztcl_Config *	cfg;
     Tcl_Obj **		dstObjVar;
{
  int		bzerror;
  More_Block	source, target;
  Tcl_Obj *	dstObj = NULL;


  source.ptr = Tcl_GetByteArrayFromObj(srcObj, &source.len);
  if (! source.len)
    {
      *dstObjVar = Tcl_NewByteArrayObj(NULL, 0);
      return NULL;
    }

  debug("request to compress %d bytes", source.len);

  target.len = source.len - (source.len >> 1);
  if (target.len < 2048)
    {
      target.len = 4096;
    }
  debug("allocating byte array of size %d", target.len);

  dstObj = Tcl_NewByteArrayObj(NULL, 0);
  bzerror = BZ_OK;
  do
    {
      if (BZ_OUTBUFF_FULL == bzerror)
	{
	  if (target.len < 4096)
	    {
	      target.len = 2*4096;
	    }
	  else if (INT_MAX == target.len)
	    {
	      bzerror = BZ_MEM_ERROR;
	      goto Error;
	    }
	  else if ((INT_MAX - target.len) < target.len)
	    {
	      target.len = INT_MAX;
	    }
	  else
	    {
	      target.len += target.len >> 1;
	    }
	  debug("reallocating with length %d", target.len);
	}
      else if (BZ_OK != bzerror)
	{
	  goto Error;
	}

      target.ptr = Tcl_SetByteArrayLength(dstObj, target.len);
      bzerror = BZ2_bzBuffToBuffCompress(target.ptr, &target.len,
					 source.ptr, (unsigned int)source.len,
					 cfg->blockSize, 0, cfg->workFactor);
    }
  while (BZ_OK != bzerror);

  debug("shrinking byte array to %d", target.len);
  Tcl_SetByteArrayLength(dstObj, target.len);
  *dstObjVar = dstObj;
  return NULL;

 Error:
  if (dstObj)
    {
      Tcl_IncrRefCount(dstObj);
      Tcl_DecrRefCount(dstObj);
    }
  return MakeErrorDescriptor(bzerror);
}

/* Bztcl_UncompressObj --

	Decompresses data in a Tcl_Obj, seen as a byte array.

   Arguments:

	srcObj -	Pointer to the object that holds the
			compressed data.
	cfg -		Pointer to the configuration for decompression.
	size -		The first guess for the size of the target buffer,
			or an integer less than zero.
	dstObjVar -	Pointer to a variable that will hold the new
			object with the uncompressed data.

   Results:

	Returns NULL or an error descriptor.

*/

More_Error
Bztcl_UncompressObj (srcObj, cfg, size, dstObjVar)
     Tcl_Obj *		srcObj;
     Bztcl_Config *	cfg;
     int		size;
     Tcl_Obj **		dstObjVar;
{
  int		bzerror, delta;
  More_Block	source, target;
  Tcl_Obj *	dstObj = NULL;


  source.ptr = Tcl_GetByteArrayFromObj(srcObj, &source.len);
  if (! source.len)
    {
      *dstObjVar = Tcl_NewByteArrayObj(NULL, 0);
      return NULL;
    }

  debug("request to decompress %d bytes", source.len);
  debug("suggested size %d, small model %d", size, small);

  if (size > 0)
    {
      target.len = size;
    }
  else
    {
      delta = source.len >> 1;
      target.len = ((INT_MAX-delta) > source.len)? (source.len+delta) : INT_MAX;
      if (target.len < 2048)
	{
	  target.len = 4096;
	}
    }
  debug("allocating byte array of size %d", target.len);

  dstObj = Tcl_NewByteArrayObj(NULL, 0);
  bzerror = BZ_OK;
  do
    {
      if (BZ_OUTBUFF_FULL == bzerror)
	{
	  if (target.len < 4096)
	    {
	      target.len = 2*4096;
	    }
	  else if (INT_MAX == target.len)
	    {
	      bzerror = BZ_MEM_ERROR;
	      goto Error;
	    }
	  else if ((INT_MAX - target.len) < target.len)
	    {
	      target.len = INT_MAX;
	    }
	  else
	    {
	      target.len <<= 1;
	    }
	  debug("reallocating byte array of size %d", target.len);
	}
      else if (BZ_OK != bzerror)
	{
	  goto Error;
	}

      target.ptr = Tcl_SetByteArrayLength(dstObj, target.len);
      bzerror = BZ2_bzBuffToBuffDecompress(target.ptr, &target.len,
					   source.ptr, (unsigned int)source.len,
					   cfg->small, 0);
    }
  while (BZ_OK != bzerror);

  debug("shrinking byte array to %d", target.len);
  Tcl_SetByteArrayLength(dstObj, target.len);
  *dstObjVar = dstObj;
  return NULL;

 Error:
  if (dstObj)
    {
      Tcl_IncrRefCount(dstObj);
      Tcl_DecrRefCount(dstObj);
    }
  return MakeErrorDescriptor(bzerror);
}

/* Bztcl_Open --

	Opens a BZ file. Actually opens only the file using the standard
	C stream  function; the  BZ stream will  be opened at  the first
	write or read operation.

   Arguments:

	fileName -	Pointer to a string representing the file
			name.
	openMode -	TCL_READABLE or TCL_WRITABLE.
	tokenVar -	Pointer to a variable that will hold the
			file descriptor.

   Results:

	Returns  NULL or  an error  descriptor.  In  case of  error: the
        integer  error specific  value is  set  to the  return value  of
        Tcl_GetErrno(), or an appropriate value.

*/

More_Error
Bztcl_Open (CONST char *fileName, int openMode, Bztcl_File *tokenVar)
{
  FILE *	file;
  Channel *	descriptor;


  openMode &= (TCL_READABLE | TCL_WRITABLE);
  debug("request to open \"%s\", readable %s, writable %s", fileName,
	question(openMode & TCL_READABLE),
	question(openMode & TCL_WRITABLE));

  Tcl_SetErrno(0);
  file = fopen(fileName, ((openMode & TCL_READABLE)? "r" : "w"));
  if (! file)
    {
      return More_ErrorErrno();
    }
  
  More_AllocStruct(descriptor, Channel);
  descriptor->file	= file;
  descriptor->openMode	= openMode;
  descriptor->config	= FileDefaultConfig;
  *tokenVar = (Bztcl_File) descriptor;
  return NULL;
}

/* ReadOpen --

	Actually opens  the BZ stream.  This function is invoked  at the
	first Bztcl_Read() invocation on a file.

   Arguments:

	descriptor -	The file descriptor.

   Results:

	Returns  NULL or  an error  descriptor.  In  case of  error: the
        integer  error specific  value is  set  to the  return value  of
        Tcl_GetErrno(), or an appropriate value.

*/

static More_Error
ReadOpen (Channel *descriptor)
{
  int		bzerror;
  BZFILE *	bzfile;


  debug("opening BZip2 stream for reading, small model %d",
	descriptor->config.small);
  bzfile = BZ2_bzReadOpen(&bzerror, descriptor->file,
			  descriptor->config.small, 0, NULL, 0);
  if (BZ_OK == bzerror)
    {
      BZ2_bzReadClose(&bzerror, descriptor->bzfile);
      descriptor->bzfile = bzfile;
      return NULL;
    }
  else
    {
      return MakeErrorDescriptor(bzerror);
    }
}

/* WriteOpen --

	Actually opens the  BZ stream.  This function is  invoked at the
	first Bztcl_Write() invocation on a file.

   Arguments:

	descriptor -	The file descriptor.

   Results:

	Returns  NULL or  an error  descriptor.  In  case of  error: the
        integer  error specific  value is  set  to the  return value  of
        Tcl_GetErrno(), or an appropriate value.

*/

static More_Error
WriteOpen (Channel *descriptor)
{
  int		bzerror;
  BZFILE *	bzfile;
  

  debug("opening BZip2 stream for writing, block size %d, work factor %d",
	descriptor->config.blockSize, descriptor->config.workFactor);
  bzfile = BZ2_bzWriteOpen(&bzerror, descriptor->file,
			   descriptor->config.blockSize, 0,
			   descriptor->config.workFactor);
  if (BZ_OK == bzerror)
    {
      BZ2_bzWriteClose(&bzerror, descriptor->bzfile, 0, NULL, NULL);
      debug("success");
      descriptor->bzfile = bzfile;
      return NULL;
    }
  else
    {
      return MakeErrorDescriptor(bzerror);
    }
}

/* Bztcl_Close --

	Closes a BZ file.

   Arguments:

   	token -		The file descriptor.

   Results:

	Returns  NULL or  an error  descriptor.  In  case of  error: the
        integer  error specific  value is  set  to the  return value  of
        Tcl_GetErrno(), or an appropriate value.

   Side effects:

        It's safe to invoke this  function multiple times for an already
        closed BZTCL descriptor: this  is to allow the invoking function
        to retry to close the FILE stream.

*/

More_Error
Bztcl_Close (Bztcl_File token)
{
  Channel *	descriptor = (Channel *) token;
  int		bzerror, unused1, unused2;


  if (descriptor->bzfile)
    {
      debug("closing BZip2 stream");

      if (TCL_READABLE == descriptor->openMode)
	{
	  BZ2_bzReadClose(&bzerror, descriptor->bzfile);
	}
      else
	{
	  BZ2_bzWriteClose(&bzerror, descriptor->bzfile, 0, &unused1, &unused2);
	  debug("bytes in %d, bytes out %d", unused1, unused2);
	}

      if (BZ_OK == bzerror)
	{
	  descriptor->bzfile = NULL;
	  debug("success");
	}
      else
	{
	  return MakeErrorDescriptor(bzerror);
	}
    }

  debug("closing C stream");

  Tcl_SetErrno(0);
  clearerr(descriptor->file);
  bzerror = fclose(descriptor->file);
  if (! bzerror)
    {
      debug("success");
      ckfree((char *) descriptor);
      return NULL;      
    }
  else
    {
      debug("error: %s", Tcl_ErrnoMsg(Tcl_GetErrno()));
      return More_ErrorErrno();
    }
}

/* Bztcl_ReadObj --

	Uncompresses data  from a  file into a  Tcl_Obj, seen as  a byte
	array. The file must have been opened for reading.

   Arguments:

	token -		The file descriptor.
	number -	Number of bytes to read.
	objVar -	Pointer to the variable that will hold the
			uncompressed data.

   Results:

	Returns  NULL or  an error  descriptor.  In  case of  error: the
        integer  error specific  value is  set  to the  return value  of
        Tcl_GetErrno(), or an appropriate value.

*/

More_Error
Bztcl_ReadObj (Bztcl_File token, int number, Tcl_Obj **objVar)
{
  More_Block	block;
  More_Error	error = NULL;
  Tcl_Obj *	objPtr;
  

  objPtr = Tcl_NewByteArrayObj(NULL, 0);

  block.len = number;
  block.ptr = Tcl_SetByteArrayLength(objPtr, block.len);

  error = Bztcl_Read(token, &block);
  if (! error)
    {
      if (block.len < number)
	{
	  Tcl_SetByteArrayLength(objPtr, block.len);
	}
      *objVar = objPtr;
    }
  else
    {
      Tcl_IncrRefCount(objPtr);
      Tcl_DecrRefCount(objPtr);
    }
  return error;
}


/* Bztcl_Read --

	Reads a data from a BZ  file and stores the uncompressed data in
	a memory block.  If an  end-of-stream is reached, the EOF status
	is recorded in the file  descriptor (so that it can be retrieved
	with Bztcl_Eof()) and the function returns no error.

   Arguments:

	descriptor -	The file descriptor.
	blockPtr -	Pointer to the target block. Before the call
			the length field holds the requested number of
			bytes; after the call, the length field holds
			the number of byte actually read.

   Results:

	Returns  NULL or  an error  descriptor.  In  case of  error: the
        integer  error specific  value is  set  to the  return value  of
        Tcl_GetErrno(), or an appropriate value.

*/

More_Error
Bztcl_Read (Bztcl_File token, More_Block *blockPtr)
{
  Channel *	descriptor = (Channel *) token;
  int		bzerror;


  if (! descriptor->bzfile)
    {
      More_Error	error;

      error = ReadOpen(descriptor);
      if (error)
	{
	  return error;
	}
    }

  if (descriptor->eof)
    {
      blockPtr->len =0;
      return NULL;
    }
  else
    {
      blockPtr->len = BZ2_bzRead(&bzerror, descriptor->bzfile,
				 blockPtr->ptr, blockPtr->len);
      if (BZ_OK == bzerror)
	{
	  debug("read %d bytes", blockPtr->len);
	  return NULL;
	}
      else if (BZ_STREAM_END == bzerror)
	{
	  debug("read %d bytes, found EOF", blockPtr->len);
	  descriptor->eof = 1;
	  return NULL;
	}
      else
	{
	  return MakeErrorDescriptor(bzerror);
	}
    }
}

/* Bztcl_WriteObj --

	Compresses data in  a Tcl_Obj, seen as a  byte array, and writes
	it in a BZ file. The file must have been opened for writing.

   Arguments:

	token -		The file descriptor.
	srcObj -	Pointer to the source object.

   Results:

	Returns  NULL or  an error  descriptor.  In  case of  error: the
        integer  error specific  value is  set  to the  return value  of
        Tcl_GetErrno(), or an appropriate value.

*/

More_Error
Bztcl_WriteObj (Bztcl_File token, Tcl_Obj *srcObj)
{
  More_Block	block;
  

  block.ptr = Tcl_GetByteArrayFromObj(srcObj, &block.len);
  return Bztcl_Write(token, block);
}

/* Bztcl_Write --

	Writes a block of data into a BZ file.

   Arguments:

	token -		The file descriptor.
	block -		The source data block.

   Results:

        Returns  NULL or  an error  descriptor.  In  case of  error: the
        integer  error specific  value is  set  to the  return value  of
        Tcl_GetErrno(), or an appropriate value.

*/

More_Error
Bztcl_Write (Bztcl_File token, More_Block block)
{
  Channel *	descriptor = (Channel *) token;
  int		bzerror;


  if (! descriptor->bzfile)
    {
      More_Error	error;

      error = WriteOpen(descriptor);
      if (error)
	{
	  return error;
	}
    }

  debug("writing %d bytes", block.len);
  BZ2_bzWrite(&bzerror, descriptor->bzfile, block.ptr, block.len);
  if (BZ_OK == bzerror)
    {
      return NULL;
    }
  else
    {
      return MakeErrorDescriptor(bzerror);
    }
}

/* Bztcl_Eof --

	Queries the state of the descriptor to see if an EOF happened on
	the last read operation.

   Arguments:

	token -		The file descriptor.

   Results:

        Returns true or false.

*/

int
Bztcl_Eof (Bztcl_File token)
{
  Channel *	descriptor = (Channel *) token;
  return (descriptor->eof);
}

/* Bztcl_GetHandle --

	Accesses the operative system handle associated to the file.

   Arguments:

	token -		The file descriptor.

   Results:

        Returns the handle.

*/

int
Bztcl_GetHandle (Bztcl_File token)
{
  Channel *	descriptor = (Channel *) token;
  return fileno(descriptor->file);
}

/* Bztcl_SetSmall --

	Configures  a new  "small" value  for a  file  descriptor.  This
	function must  be used only  to configure a  readable descriptor
	prior to the first invocation to Bztcl_Read(), else it will have
	no effect.

   Arguments:

	token -		The file descriptor.
	small -		The new value.

   Results:

        Updates the descriptor state.

*/

void
Bztcl_SetSmall (Bztcl_File token, int small)
{
  Channel *	descriptor = (Channel *) token;

  if ((! descriptor->bzfile) && (descriptor->openMode & TCL_READABLE))
    {
      descriptor->config.small = small;
    }
}

/* Bztcl_SetWorkFactor --

	Configures a new work factor  value for a file descriptor.  This
	function must  be used only  to configure a  writable descriptor
	prior  to the first  invocation to  Bztcl_Write(), else  it will
	have no effect.

   Arguments:

	token -		The file descriptor.
	workFactor -	The new value.

   Results:

        Updates the descriptor state.

*/

void
Bztcl_SetWorkFactor (Bztcl_File token, int workFactor)
{
  Channel *	descriptor = (Channel *) token;

  if ((! descriptor->bzfile) && (descriptor->openMode & TCL_WRITABLE))
    {
      descriptor->config.workFactor = workFactor;
    }
}

/* Bztcl_SetBlockSize --

	Configures a new  block size value for a  file descriptor.  This
	function must  be used only  to configure a descriptor  prior to
	the first invocation to Bztcl_Write().

   Arguments:

	token -		The file descriptor.
	blockSize -	The new value.

   Results:

        Updates the descriptor state.

*/

void
Bztcl_SetBlockSize (Bztcl_File token, int blockSize)
{
  Channel *	descriptor = (Channel *) token;

  if ((! descriptor->bzfile) && (descriptor->openMode & TCL_WRITABLE))
    {
      descriptor->config.blockSize = blockSize;
    }
}

/* Bztcl_GetWorkFactor --

	Accesses the work factor configured for a descriptor.

   Arguments:

	token -		The file descriptor.

   Results:

        Returns the work factor value.

*/

int
Bztcl_GetWorkFactor (Bztcl_File token)
{
  Channel *	descriptor = (Channel *) token;
  return descriptor->config.workFactor;
}

/* Bztcl_GetBlockSize --

	Accesses the block size configured for the descriptor.

   Arguments:

	token -		The file descriptor.

   Results:

        Returns the number of blocks.

*/

int
Bztcl_GetBlockSize (Bztcl_File token)
{
  Channel *	descriptor = (Channel *) token;
  return descriptor->config.blockSize;
}

/* Bztcl_GetSmall --

	Accesses the "small" parameter.

   Arguments:

	token -		The file descriptor.

   Results:

        Returns the number of blocks.

*/

int
Bztcl_GetSmall (Bztcl_File token)
{
  Channel *	descriptor = (Channel *) token;
  return descriptor->config.small;
}

/* Bztcl_ReadableFile --

	Tests if the file was opened for reading.

   Arguments:

	token -		The file descriptor.

   Results:

        Returns true if the file is readable.

*/

int
Bztcl_ReadableFile (Bztcl_File token)
{
  Channel *	descriptor = (Channel *) token;
  return (descriptor->openMode & TCL_READABLE);
}

/* Bztcl_WritableFile --

	Tests if the file was opened for writing.

   Arguments:

	token -		The file descriptor.

   Results:

        Returns true if the file is writable.

*/

int
Bztcl_WritableFile (Bztcl_File token)
{
  Channel *	descriptor = (Channel *) token;
  return (descriptor->openMode & TCL_WRITABLE);
}

/* Bztcl_IsBlockSize --

	Validates  an   integer  that  is  candidated  to   be  used  as
	compression  block size. It  must be  a number  between 1  and 9
	inclusive  and it  expresses  the  block size  in  units of  100
	kilobytes.

   Arguments:

	blockSize -	The integer to be validated.

   Results:

        Returns true if the value is correct, else returns false.

*/

int
Bztcl_IsBlockSize (int blockSize)
{
  return ((0 <= blockSize) && (blockSize <= 9));
}

/* Bztcl_IsWorkFactor --

	Validates  an integer  that  is  candidate to  be  used as  work
	factor.   The work  factor  controls how  the compression  phase
	behaves  when presented  with the  worst  case.  It  must be  an
	integer in  the range 0-250  inclusive; 0 is interpreted  as 30:
	the default value, good in most cases.

   Arguments:

	workFactor -	The integer to be validated.

   Results:

        Returns true if the value is valid, else returns false.

*/

int
Bztcl_IsWorkFactor (int workFactor)
{
  return ((0 <= workFactor) && (workFactor <= 250));
}

/* Bztcl_GetBlockSizeFromObj --

	Extracts a  valid block  size from an  object.  "default"  is an
	acceptable string value for the input object: it means 3.

   Arguments:

	interp -	If not NULL, the interp in which
			report errors.
	objPtr -	Pointer to the source object.
	blockSizeVar -	Pointer to the target variable.

   Results:

        Returns TCL_OK if a good value was found, otherwise TCL_ERROR.

*/

int
Bztcl_GetBlockSizeFromObj (interp, objPtr, blockSizeVar)
     Tcl_Interp	*	interp;
     Tcl_Obj	*	objPtr;
     int	*	blockSizeVar;
{
  int		e, blockSize;
  
  
  e = Tcl_GetIntFromObj(interp, objPtr, &blockSize);
  if (TCL_OK == e)
    {
      if (Bztcl_IsBlockSize(blockSize))
	{
	  *blockSizeVar = blockSize;
	  return TCL_OK;
	}
      else
	{
	  if (interp)
	    {
	      Tcl_ResetResult(interp);
	      Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
				     "blocks number out of range (",
				     Tcl_GetString(objPtr),
				     ") must be 0-9", NULL);
	      Tcl_SetErrorCode(interp, "LOGIC", NULL);
	    }
	  return TCL_ERROR;
	}
    }
  else
    {
      if (strcmp("default", Tcl_GetString(objPtr)) == 0)
	{
	  Tcl_ResetResult(interp);
	  *blockSizeVar = DEFAULT_BLOCK_SIZE;
	  return TCL_OK;
	}
      else
	{
	  if (interp)
	    {
	      /*
		Append  description   to  the  error   message  left  by
		Tcl_GetIntFromObj().
	      */
	      Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
				     ": wrong blocks number", NULL);
	      Tcl_SetErrorCode(interp, "LOGIC", NULL);
	    }
	  return TCL_ERROR;
	}
    }
}


/* Bztcl_GetWorkFactorFromObj --

	Extracts a valid BZip2 work factor from an object.  "default" is
	an acceptable string value for the input object: it means 30.

   Arguments:

	interp -	If not NULL, the interp in which report errors.
	objPtr -	Pointer to the source object.
	workFactorVar - Pointer to the target variable.

   Results:

        Returns TCL_OK if a good value was found, otherwise TCL_ERROR.

*/

int
Bztcl_GetWorkFactorFromObj (interp, objPtr, workFactorVar)
     Tcl_Interp	*	interp;
     Tcl_Obj *		objPtr;
     int *		workFactorVar;
{
  int		e;
  int		workFactor;


  e = Tcl_GetIntFromObj(interp, objPtr, &workFactor);
  if (TCL_OK == e)
    {
      if (Bztcl_IsWorkFactor(workFactor))
	{
	  *workFactorVar = workFactor;
	  return TCL_OK;
	}
      else
	{
	  if (interp)
	    {
	      Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
				     "work factor out of range (",
				     Tcl_GetString(objPtr),
				     ") must be 0-250", NULL);
	    }
	  return TCL_ERROR;
	}
    }
  else
    {
      if (strcmp("default", Tcl_GetString(objPtr)) == 0)
	{
	  Tcl_ResetResult(interp);
	  *workFactorVar = DEFAULT_WORK_FACTOR;
	  return TCL_OK;
	}
      else
	{
	  if (interp)
	    {
	      /*
		Append  description   to  the  error   message  left  by
		Tcl_GetIntFromObj().
	      */
	      Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
				     ": wrong work factor", NULL);
	    }
	  return TCL_ERROR;
	}
    }
}

/* Bztcl_Version --

	Accesses the version of BZip2.

   Arguments:

	None.

   Results:

        Returns a pointer to the string representing the version.

*/

CONST char *
Bztcl_Version (void)
{
  return BZ2_bzlibVersion();
}

/* Bztcl_MakeErrorDescriptor --

	Builds an error descriptor holding informations about a BZ error
	code.

   Arguments:

	bzerror -	The error code: one of the BZ_* symbols.

   Results:

        Returns the error descriptor.

*/

More_Error
Bztcl_MakeErrorDescriptor (int bzerror)
{
  More_Error	error = More_ErrorNew();
  CONST char *	errorString;
  int		errorCode;


  debug("building error descriptor");
  assert(BZ_OK != bzerror);
  switch (bzerror)
    {
    case BZ_PARAM_ERROR:
      debug("BZ_PARAM_ERROR");
      errorString	= "invalid configuration parameter";
      errorCode		= EINVAL;
      goto Error_Logic;

    case BZ_SEQUENCE_ERROR:
      debug("BZ_SEQUENCE_ERROR");
      errorString	= "wrong file open mode";
      errorCode		= EINVAL;
      goto Error_Logic;

    case BZ_MEM_ERROR:
      debug("BZ_MEM_ERROR");
      errorString	= "not enough memory";
      errorCode		= ENOMEM;
      goto Error_Runtime;
      
    case BZ_IO_ERROR:
      debug("BZ_IO_ERROR");
      errorCode		= Tcl_GetErrno();
      errorString	= Tcl_ErrnoMsg(errorCode);
      goto Error_Runtime;
      
    case BZ_CONFIG_ERROR:
      debug("BZ_CONFIG_ERROR");
      errorString	= "library internal error";
      errorCode	= EINVAL;
      goto Error_Runtime;

    case BZ_UNEXPECTED_EOF:
    case BZ_DATA_ERROR:
    case BZ_DATA_ERROR_MAGIC:
      debug("BZ_DATA_ERROR");
      errorString	= "corrupted input data";
      errorCode	= EINVAL;
      goto Error_Runtime;

    default:
      debug("unknown internal error (%d)", bzerror);
      errorString	= "unknown internal error";
      errorCode		= EINVAL;
      goto Error_Logic;
    }
  
 Error_Logic:
  More_ErrorLogicStr(error, errorString);
  goto Error;
 Error_Runtime:
  More_ErrorRuntimeStr(error, errorString);
 Error:
  More_ErrorSetInt(error, errorCode);
  return error;
}


/* end of file */
/*
Local Variables:
mode: c
page-delimiter: "^$"
End:
*/
