/* stream.c --
   
   Part of: bztcl
   Contents: stream implementation
   Date: Fri Feb 20, 2004
   
   Abstract
   
   
   
   Copyright (c) 2004 Marco Maggi
   
   The author  hereby grant permission to use,  copy, modify, distribute,
   and  license this  software  and its  documentation  for any  purpose,
   provided that  existing copyright notices  are retained in  all copies
   and that  this notice  is included verbatim  in any  distributions. No
   written agreement, license, or royalty  fee is required for any of the
   authorized uses.  Modifications to this software may be copyrighted by
   their authors and need not  follow the licensing terms described here,
   provided that the new terms are clearly indicated on the first page of
   each file where they apply.
   
   IN NO  EVENT SHALL THE AUTHOR  OR DISTRIBUTORS BE LIABLE  TO ANY PARTY
   FOR  DIRECT, INDIRECT, SPECIAL,  INCIDENTAL, OR  CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE  USE OF THIS  SOFTWARE, ITS DOCUMENTATION,  OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR  HAVE BEEN  ADVISED OF  THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR  AND DISTRIBUTORS  SPECIFICALLY  DISCLAIM ANY  WARRANTIES,
   INCLUDING,   BUT   NOT  LIMITED   TO,   THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A    PARTICULAR   PURPOSE,    AND
   NON-INFRINGEMENT.  THIS  SOFTWARE IS PROVIDED  ON AN "AS  IS" BASIS,
   AND  THE  AUTHOR  AND  DISTRIBUTORS  HAVE  NO  OBLIGATION  TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
   
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#if 0
#  define TCLMORE_DEBUG_MESSAGES
#  define TCLMORE_DEBUGGING	1
#endif

#include <assert.h>
#include <errno.h>
#include <stdarg.h>
#include "bztclInt.h"
#include "bzlib.h"


/** ------------------------------------------------------------
 ** Preprocessor symbols.
 ** ----------------------------------------------------------*/

#define debug			More_Debug
#define question		More_Question

#define OUTPUT_BUFFER_SIZE		(4096*4)
#define OUTPUT_BUFFER_HYSTERESIS	(4096*4)


#define OutputBufferGetNumberOfUsedBytes(stream)		\
	((stream)->output.allocated.len - (stream)->bzstream.context.avail_out)
#define OutputBufferIsAllocated(stream)	(NULL != (stream)->output.allocated.ptr)
#define OutputBufferFree(stream)	ckfree((char *)(stream)->output.allocated.ptr)

#define MakeErrorDescriptor	Bztcl_MakeErrorDescriptor


/* bz context macros */

#define ContextInputDataIsAvailable(context)	(context->avail_in)
#define ContextUnusedLength(context)		(context->avail_out)


/** ------------------------------------------------------------
 ** Debug message macros.
 ** ----------------------------------------------------------*/

#define Debug_PrintStreamType(stream)		\
	debug("%s stream (stream=%p)",	\
		((stream)->driver == &CompressionDriver)? "compress" : "decompress", \
		(stream))

#define Debug_PrintAvailableData(stream, prefix)	\
	debug("%s: avail_in %d, output buffer: size=%d, used=%d, avail_out=%d",	\
		(prefix),					\
		(stream)->bzstream.context.avail_in,		\
		(stream)->output.allocated.len,			\
		OutputBufferGetNumberOfUsedBytes(stream),	\
		(stream)->bzstream.context.avail_out)


/** ------------------------------------------------------------
 ** Assertion macros.
 ** ----------------------------------------------------------*/

#define Assert_ValidateOutputBuffer(stream)		\
	assert((0 == (stream)->bzstream.context.avail_out) || \
               (((stream)->output.allocated.ptr + (stream)->output.allocated.len) > \
	        (More_BytePtr)(stream)->bzstream.context.next_out));		\
	assert((More_BytePtr)((stream)->bzstream.context.next_out +		\
		(stream)->bzstream.context.avail_out) == \
               ((stream)->output.allocated.ptr + (stream)->output.allocated.len))

#define Assert_OffsetInOutputBuffer(stream, offset)	\
	assert((stream)->output.allocated.len - \
		(stream)->bzstream.context.avail_out >= (offset))


/** ------------------------------------------------------------
 ** Module's type declarations.
 ** ----------------------------------------------------------*/

typedef struct Stream		Stream;
typedef struct BzStream		BzStream;


typedef int DriverInit		_ANSI_ARGS_((BzStream *bzstream));
typedef int DriverWrite		_ANSI_ARGS_((Stream *stream));
typedef int DriverFlush		_ANSI_ARGS_((Stream *stream));
typedef int DriverFinish	_ANSI_ARGS_((Stream *stream));

typedef struct Driver {
  DriverInit *			init;
  DriverWrite *			write;
  DriverFlush *			flush;
  DriverFinish *		finish;
} Driver;


struct BzStream {
  bz_stream		context;
  Bztcl_Config		config;
};

typedef struct OutputBuffer {
  More_Block		allocated;
  int			size;
  int			hysteresis;
} OutputBuffer;

struct Stream {
  BzStream		bzstream;
  Driver *		driver;
  OutputBuffer		output;
  int			finished;
};


/** ------------------------------------------------------------
 ** Modules functions prototypes.
 ** ----------------------------------------------------------*/

static DriverInit	CompressInit;
static DriverWrite	CompressWrite;
static DriverFlush	CompressFlush;
static DriverFinish	CompressFinish;

static DriverInit	DecompressInit;
static DriverWrite	DecompressWrite;
static DriverFlush	DecompressFlush;
static DriverFinish	DecompressFinish;

static int DecompressCheckEndOfStream	_ANSI_ARGS_((Stream *stream,
						     int bzerror));

static void OutputBufferEnlarge	_ANSI_ARGS_((Stream *stream));
static void OutputBufferShrink	_ANSI_ARGS_((Stream *stream));


static void * AllocFunc _ANSI_ARGS_((void *opaque, int number, int size));
static void FreeFunc	_ANSI_ARGS_((void *opaque, void *block));

static More_Error RuntimeError	_ANSI_ARGS_((CONST char *errorInfo,
					     int errorCode));


EXTERN More_Error Bztcl_MakeErrorDescriptor _ANSI_ARGS_((int bzerror));


/** ------------------------------------------------------------
 ** Module's variables.
 ** ----------------------------------------------------------*/

static Driver CompressionDriver = {
  CompressInit,
  CompressWrite,
  CompressFlush,
  CompressFinish
};

static Driver DecompressionDriver = {
  DecompressInit,
  DecompressWrite,
  DecompressFlush,
  DecompressFinish
};


More_Error
Bztcl_StreamInit (Bztcl_Stream *tokenVar, int compression, Bztcl_Config *config)
{
  Stream *	stream;
  More_Error	error = NULL;
  int		bzerror;


  debug("creating stream, compression? %s", question(compression));

  More_AllocStruct(stream, Stream);
  stream->driver = (compression)? &CompressionDriver : &DecompressionDriver;
  stream->output.size			= OUTPUT_BUFFER_SIZE;
  stream->output.hysteresis		= OUTPUT_BUFFER_HYSTERESIS;
  stream->bzstream.context.bzalloc	= AllocFunc;
  stream->bzstream.context.bzfree	= FreeFunc;
  stream->bzstream.config		= *config;

  bzerror = stream->driver->init(&(stream->bzstream));
  if (BZ_OK == bzerror)
    {
      *tokenVar = (Bztcl_Stream) stream;
    }
  else
    {
      ckfree((char *) stream);
      error = MakeErrorDescriptor(bzerror);
    }
  return error;
}
static int
CompressInit (BzStream *bzstream)
{
  return BZ2_bzCompressInit(&(bzstream->context), bzstream->config.blockSize,
			    0, bzstream->config.workFactor);
}
static int
DecompressInit (BzStream *bzstream)
{
  return BZ2_bzDecompressInit(&(bzstream->context), 0, bzstream->config.small);
}

void
Bztcl_StreamFinal (Bztcl_Stream token)
{
  Stream *	stream = (Stream *) token;


  debug("finalising stream, to finish? %s", question(! stream->finished));
  if (! stream->finished)
    {
      stream->driver->finish(stream);
    }
  if (OutputBufferIsAllocated(stream))
    {
      OutputBufferFree(stream);
    }
  ckfree((char *) stream);
}

More_Block
Bztcl_StreamOutput (Bztcl_Stream token)
{
  Stream *	stream = (Stream *) token;
  More_Block	block;


  block.len = OutputBufferGetNumberOfUsedBytes(stream);
  block.ptr = stream->output.allocated.ptr;
  return block;
}

void
Bztcl_StreamRead (Bztcl_Stream token, int numberOfReadBytes)
{
  Stream *	stream		= (Stream *) token;
  More_Block *	outputBuffer	= &(stream->output.allocated);
  bz_stream *	context		= &(stream->bzstream.context);
  size_t	numberOfUnreadBytes;


  Assert_ValidateOutputBuffer(stream);
  Assert_OffsetInOutputBuffer(stream, numberOfReadBytes);
  debug("commit read: %d bytes were read", numberOfReadBytes);

  /*
                              outputBuffer->len
    |-------------------------------------------------------------------|
    <...................><___________________><.........................>
      numberOfReadBytes   numberOfUnreadBytes      context->avail_out
  */

  numberOfUnreadBytes = \
    outputBuffer->len - numberOfReadBytes - context->avail_out;

  memmove(outputBuffer->ptr, outputBuffer->ptr + numberOfReadBytes,
	  numberOfUnreadBytes);

  context->next_out  -= numberOfReadBytes;
  context->avail_out += numberOfReadBytes;

  Assert_ValidateOutputBuffer(stream);
  Debug_PrintAvailableData(stream, "after reading data");
  OutputBufferShrink(stream);
}

More_Error
Bztcl_StreamWrite (Bztcl_Stream token, More_Block *inputPtr)
{
  Stream *	stream = (Stream *) token;
  bz_stream *	context = &(stream->bzstream.context);
  int		bzerror;
  More_Error	error  = NULL;


  if ((inputPtr->len > 0) && (! stream->finished))
    {
      Debug_PrintStreamType(stream);
      debug("request to write %d bytes", inputPtr->len);
      context->next_in  = inputPtr->ptr;
      context->avail_in = inputPtr->len;
      bzerror = stream->driver->write(stream);
      if (BZ_OK == bzerror)
	{

	  if (ContextInputDataIsAvailable(context))
	    {
	      inputPtr->len = context->avail_in;
	      inputPtr->ptr = context->next_in;
	      debug("unread input %d bytes", inputPtr->len);
	    }
	  else
	    {
	      More_BlockReset(*inputPtr);
	    }
	  debug("end operation");
	}
      else
	{
	  debug("write error: %d", bzerror);
	  error = MakeErrorDescriptor(bzerror);
	}
      context->next_in  = NULL;
      context->avail_in = 0;
    }
  Debug_PrintAvailableData(stream, "after writing");
  return error;
}

static int
CompressWrite (Stream *stream)
{
  int		bzerror;
  bz_stream *	context = &(stream->bzstream.context);


  debug("compress write loop");
  do
    {
      OutputBufferEnlarge(stream);
      bzerror = BZ2_bzCompress(context, BZ_RUN);
    }
  while (ContextInputDataIsAvailable(context) && (BZ_RUN_OK == bzerror));
  return (BZ_RUN_OK == bzerror)? BZ_OK : bzerror;
}

static int
DecompressWrite (Stream *stream)
{
  int		bzerror;
  bz_stream *	context = &(stream->bzstream.context);


  debug("decompress write loop");
  do
    {
      OutputBufferEnlarge(stream);
      bzerror = BZ2_bzDecompress(context);
    }
  while (ContextInputDataIsAvailable(context) && (BZ_OK == bzerror));
  return DecompressCheckEndOfStream(stream, bzerror);
}

More_Error
Bztcl_StreamFinish (Bztcl_Stream token)
{
  Stream *	stream = (Stream *) token;
  int		bzerror;


  if (! stream->finished)
    {
      stream->finished = 1;
      bzerror = stream->driver->finish(stream);
      debug("after finishing: code %d", bzerror);
      if (BZ_STREAM_END != bzerror)
	{
	  return (BZ_OK == bzerror)? \
	    RuntimeError("end of stream not found", EINVAL) : \
	    MakeErrorDescriptor(bzerror);
	}
      Debug_PrintAvailableData(stream, "after finishing");
      Assert_ValidateOutputBuffer(stream);
    }
  return NULL;
}

static int
CompressFinish (Stream *stream)
{
  int		bzerror;
  bz_stream *	context = &(stream->bzstream.context);


  do
    {
      OutputBufferEnlarge(stream);
      bzerror = BZ2_bzCompress(context, BZ_FINISH);
    }
  while (BZ_FINISH_OK == bzerror);
  BZ2_bzCompressEnd(context);
  return bzerror;
}

static int
DecompressFinish (Stream *stream)
{
  int		bzerror, outputAvailableBefore, outputAvailableAfter;
  bz_stream *	context = &(stream->bzstream.context);


  debug("finishing decompress stream");
  do
    {
      OutputBufferEnlarge(stream);
      outputAvailableBefore = context->avail_out;
      bzerror = BZ2_bzDecompress(context);
      outputAvailableAfter  = context->avail_out;
      debug("input data: before %d, after %d",
	    outputAvailableBefore, outputAvailableAfter);
    }
  while ((BZ_OK == bzerror) && (outputAvailableBefore < outputAvailableAfter));
  BZ2_bzDecompressEnd(context);
  return bzerror;
}

More_Error
Bztcl_StreamFlush (Bztcl_Stream token)
{
  Stream *	stream  = (Stream *) token;
  int		bzerror = BZ_OK;


  if (! stream->finished)
    {
      bzerror = stream->driver->flush(stream);
      Debug_PrintAvailableData(stream, "after flushing");
      Assert_ValidateOutputBuffer(stream);
    }
  return (BZ_OK == bzerror)? NULL : MakeErrorDescriptor(bzerror);
}

static int
CompressFlush (Stream *stream)
{
  int		bzerror;
  bz_stream *	context = &(stream->bzstream.context);


  debug("compression flushing");
  do
    {
      OutputBufferEnlarge(stream);
      bzerror = BZ2_bzCompress(context, BZ_FLUSH);
      Debug_PrintAvailableData(stream, "flushed block");
    }
  while (BZ_FLUSH_OK == bzerror);
  /* normalise return value */
  return (BZ_RUN_OK == bzerror)? BZ_OK : bzerror;
}

static int
DecompressFlush (Stream *stream)
{
  int		bzerror, availableOutputBefore, availableOutputAfter;
  bz_stream *	context = &(stream->bzstream.context);


  debug("decompression flushing");
  do
    {
      OutputBufferEnlarge(stream);
      availableOutputBefore = context->avail_out;
      bzerror = BZ2_bzDecompress(context);
      availableOutputAfter = context->avail_out;
      Debug_PrintAvailableData(stream, "flushed block");
    }
  while ((BZ_OK == bzerror) && (availableOutputBefore < availableOutputAfter));
  return DecompressCheckEndOfStream(stream, bzerror);
}

static int
DecompressCheckEndOfStream (Stream *stream, int bzerror)
{
  if (BZ_STREAM_END == bzerror)
    {
      debug("finished decompression stream");
      BZ2_bzDecompressEnd(&(stream->bzstream.context));
      stream->finished = 1;
      bzerror = BZ_OK;
    }
  return bzerror;
}

void
Bztcl_StreamSetBufferSize (Bztcl_Stream token, int size)
{
  Stream *	stream = (Stream *) token;

  stream->output.size = size;
  if (OutputBufferIsAllocated(stream) &&
      (OutputBufferGetNumberOfUsedBytes(stream) < size))
    {
      More_BlockRealloc(stream->output.allocated, size);
    }
}
void
Bztcl_StreamSetBufferHysteresis (Bztcl_Stream token, int hysteresis)
{
  Stream *	stream = (Stream *) token;
  stream->output.hysteresis = hysteresis;
}
int
Bztcl_StreamGetBufferSize (Bztcl_Stream token)
{
  Stream *	stream = (Stream *) token;
  return (stream->output.allocated.len);
}
int
Bztcl_StreamGetBufferHysteresis (Bztcl_Stream token)
{
  Stream *	stream = (Stream *) token;
  return (stream->output.hysteresis);
}
int
Bztcl_StreamBufferAllocated (Bztcl_Stream token)
{
  Stream *	stream = (Stream *) token;
  return OutputBufferIsAllocated(stream);
}

int
Bztcl_StreamGetWorkFactor (Bztcl_Stream token)
{
  Stream *	stream = (Stream *) token;
  return (stream->bzstream.config.workFactor);
}
int
Bztcl_StreamGetBlockSize (Bztcl_Stream token)
{
  Stream *	stream = (Stream *) token;
  return (stream->bzstream.config.blockSize);
}
int
Bztcl_StreamGetSmall (Bztcl_Stream token)
{
  Stream *	stream = (Stream *) token;
  return (stream->bzstream.config.small);
}
int
Bztcl_StreamFinished (Bztcl_Stream token)
{
  Stream *	stream = (Stream *) token;
  return (stream->finished);
}

static void
OutputBufferEnlarge (Stream *stream)
{
  int		hysteresis	= stream->output.hysteresis;
  bz_stream *	context		= &(stream->bzstream.context);
  More_Block *	outputBuffer	= &(stream->output.allocated);


  Debug_PrintAvailableData(stream, "before enlargement");
  if (! outputBuffer->len)
    {
      More_BlockAlloc(*outputBuffer, stream->output.size);
      context->next_out  = outputBuffer->ptr;
      context->avail_out = outputBuffer->len;
    }
  else if (ContextUnusedLength(context) < hysteresis)
    {
      size_t	newSize   = outputBuffer->len;
      ptrdiff_t	usedSpace = OutputBufferGetNumberOfUsedBytes(stream);

      newSize += (newSize % hysteresis) + hysteresis;
      More_BlockRealloc(*outputBuffer, newSize);
      context->next_out  = outputBuffer->ptr + usedSpace;
      context->avail_out = newSize - usedSpace;
    }
  Debug_PrintAvailableData(stream, "after  enlargement");
  Assert_ValidateOutputBuffer(stream);
}

static void
OutputBufferShrink (Stream *stream)
{
  int		hysteresis	= stream->output.hysteresis;
  bz_stream *	context		= &(stream->bzstream.context);
  More_Block *	outputBuffer	= &(stream->output.allocated);


  if (ContextUnusedLength(context) > (2*hysteresis))
    {
      int	usedSpace = OutputBufferGetNumberOfUsedBytes(stream);
      int	newSize   = usedSpace + usedSpace % hysteresis + hysteresis;

      More_BlockRealloc(*outputBuffer, newSize);
      context->next_out  = outputBuffer->ptr + usedSpace;
      context->avail_out = outputBuffer->len - usedSpace;
    }
  Debug_PrintAvailableData(stream, "shrinked buffer");
  Assert_ValidateOutputBuffer(stream);
}

static void *
AllocFunc (void *opaque, int number, int size)
{
  return ((void *) ckalloc((unsigned) (number * size)));
}

static void
FreeFunc (void *opaque, void *block)
{
  ckfree((char *) block);
}

static More_Error
RuntimeError (CONST char *errorInfo, int errorCode)
{
  More_Error error;


  error = More_ErrorNew();
  More_ErrorRuntimeStr(error, errorInfo);
  More_ErrorSetInt(error, errorCode);
  return error;
}


/* end of file */
/*
Local Variables:
mode: c
page-delimiter: "^$"
End:
*/
