/* transformation.c --
   
   Part of: BZTCL
   Contents: transformation driver
   Date: Thu Feb 19, 2004
   
   Abstract
   
	This  module  implements the  BZ  compression and  decompression
	transformation:  a  stream that  can  be  stacked upon  existing
	channels.
   
   Copyright (c) 2004 Marco Maggi
   
   The author  hereby grant permission to use,  copy, modify, distribute,
   and  license this  software  and its  documentation  for any  purpose,
   provided that  existing copyright notices  are retained in  all copies
   and that  this notice  is included verbatim  in any  distributions. No
   written agreement, license, or royalty  fee is required for any of the
   authorized uses.  Modifications to this software may be copyrighted by
   their authors and need not  follow the licensing terms described here,
   provided that the new terms are clearly indicated on the first page of
   each file where they apply.
   
   IN NO  EVENT SHALL THE AUTHOR  OR DISTRIBUTORS BE LIABLE  TO ANY PARTY
   FOR  DIRECT, INDIRECT, SPECIAL,  INCIDENTAL, OR  CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE  USE OF THIS  SOFTWARE, ITS DOCUMENTATION,  OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR  HAVE BEEN  ADVISED OF  THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR  AND DISTRIBUTORS  SPECIFICALLY  DISCLAIM ANY  WARRANTIES,
   INCLUDING,   BUT   NOT  LIMITED   TO,   THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A    PARTICULAR   PURPOSE,    AND
   NON-INFRINGEMENT.  THIS  SOFTWARE IS PROVIDED  ON AN "AS  IS" BASIS,
   AND  THE  AUTHOR  AND  DISTRIBUTORS  HAVE  NO  OBLIGATION  TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
   
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#if 0
#  define TCLMORE_DEBUG_MESSAGES
#  define TCLMORE_DEBUGGING	1
#endif

#include "bztclInt.h"


/** ------------------------------------------------------------
 ** Preprocessor symbols.
 ** ----------------------------------------------------------*/

#define debug			More_Debug
#define question		More_Question

#define InputStreamExists(T)	((T)->input)
#define OutputStreamExists(T)	((T)->output)



/** ------------------------------------------------------------
 ** Module's function prototypes.
 ** ----------------------------------------------------------*/

static More_ChannelDriverSetOptionProc SetOptionBufferSize;
static More_ChannelDriverSetOptionProc SetOptionBufferDelta;

static More_ChannelDriverGetOptionProc GetOptionBufferSize;
static More_ChannelDriverGetOptionProc GetOptionBufferDelta;

static int GetIntFromString	_ANSI_ARGS_((Tcl_Interp *interp,
					     CONST char *string,
					     int *valueVar));


/** ------------------------------------------------------------
 ** Module's variables.
 ** ----------------------------------------------------------*/

static CONST More_ChannelDriverOption optionTable[] = {
  { "-inputBufferSize",   SetOptionBufferSize,  GetOptionBufferSize },
  { "-outputBufferSize",  SetOptionBufferSize,  GetOptionBufferSize },
  { "-inputBufferDelta",  SetOptionBufferDelta, GetOptionBufferDelta },
  { "-outputBufferDelta", SetOptionBufferDelta, GetOptionBufferDelta },
  { NULL, NULL, NULL }
};

static CONST More_StreamDriver Bztcl_StreamDriver = {
  "1",
  "bztcl",
  Bztcl_StreamFinal,
  Bztcl_StreamOutput,
  Bztcl_StreamRead,
  Bztcl_StreamWrite,
  Bztcl_StreamFlush,
  Bztcl_StreamFinish,
  optionTable
};


Tcl_Channel
Bztcl_MakeTransform (input, output, subChannel)
     Bztcl_Stream	input;
     Bztcl_Stream	output;
     Tcl_Channel	subChannel;
{
  debug("input %p, output %p", input, output);
  return More_MakeStreamTransform(&Bztcl_StreamDriver,
				  input, output, subChannel);
}

static int
SetOptionBufferSize (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     CONST char *	optionValue;
{
  More_StreamIO *	T = (More_StreamIO *) D;
  int			bufferSize, e;
  CONST char *		errorInfo;
  Bztcl_Stream		stream;


  debug("input %p, output %p", T->input, T->output);
  if (('i' == optionName[1]) && (! InputStreamExists(T)))
    {
      debug("no input stream");
      errorInfo = "this channel has no input stream";
      goto Error;
    }
  else if (('o' == optionName[1]) && (! OutputStreamExists(T)))
    {
      debug("no output stream");
      errorInfo = "this channel has no output stream";
      goto Error;
    }
  else
    {
      debug("setting option %s to %s", optionName, optionValue);
      e = GetIntFromString(interp, optionValue, &bufferSize);
      if (TCL_OK != e)
	{
	  return (interp)? More_LogicError(interp) : e;
	}
      stream = ((optionName[1] == 'i')? T->input : T->output);
      Bztcl_StreamSetBufferSize(stream, bufferSize);
    }
  return TCL_OK;

 Error:
  return More_RuntimeErrorStr(interp, errorInfo);
}

static int
SetOptionBufferDelta (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     CONST char *	optionValue;
{
  More_StreamIO *	T = (More_StreamIO *) D;
  int			hysteresis, e;
  CONST char *		errorInfo;
  Bztcl_Stream		stream;


  debug("input %p, output %p", T->input, T->output);
  if (('i' == optionName[1]) && (! InputStreamExists(T)))
    {
      errorInfo = "this channel has no input stream";
      goto Error;
    }
  else if (('o' == optionName[1]) && (! OutputStreamExists(T)))
    {
      errorInfo = "this channel has no output stream";
      goto Error;
    }
  else
    {
      e = GetIntFromString(interp, optionValue, &hysteresis);
      if (TCL_OK != e)
	{
	  return (interp)? More_LogicError(interp) : e;
	}
      stream = ((optionName[1] == 'i')? T->input : T->output);
      Bztcl_StreamSetBufferHysteresis(stream, hysteresis);
    }
  return TCL_OK;

 Error:
  return More_RuntimeErrorStr(interp, errorInfo);
}

static int
GetOptionBufferSize (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     Tcl_DString *	optionValue;
{
  More_StreamIO *	T = (More_StreamIO *) D;


  debug("input %p, output %p", T->input, T->output);
  if (('i' == optionName[1]) && (! InputStreamExists(T)))
    {
      debug("no input stream");
      Tcl_DStringAppend(optionValue, "0", -1);
    }
  else if (('o' == optionName[1]) && (! OutputStreamExists(T)))
    {
      debug("no output stream %p", T->output);
      Tcl_DStringAppend(optionValue, "0", -1);
    }
  else
    {
      int		bufferSize;
      Bztcl_Stream	stream = ((optionName[1] == 'i')? T->input : T->output);


      bufferSize = Bztcl_StreamGetBufferSize(stream);
      More_DStringPrintf(optionValue, "%d", bufferSize);
      debug("getting option %s: %d", optionName, bufferSize);
    }
  return TCL_OK;
}

static int
GetOptionBufferDelta (D, interp, optionName, optionValue)
     ClientData		D;
     Tcl_Interp *	interp;
     CONST char *	optionName;
     Tcl_DString *	optionValue;
{
  More_StreamIO *	T = (More_StreamIO *) D;


  debug("input %p, output %p", T->input, T->output);
  if (('i' == optionName[1]) && (! InputStreamExists(T)))
    {
      Tcl_DStringAppend(optionValue, "0", -1);
    }
  else if (('o' == optionName[1]) && (! OutputStreamExists(T)))
    {
      Tcl_DStringAppend(optionValue, "0", -1);
    }
  else
    {
      int		bufferDelta;
      Bztcl_Stream	stream = ((optionName[1] == 'i')? T->input : T->output);

      bufferDelta = Bztcl_StreamGetBufferHysteresis(stream);
      More_DStringPrintf(optionValue, "%d", bufferDelta);
    }
  return TCL_OK;
}

static int
GetIntFromString (Tcl_Interp *interp, CONST char *string, int *valueVar)
{
  Tcl_Obj *	objPtr;
  int		e;

  objPtr = Tcl_NewStringObj(string, -1);
  Tcl_IncrRefCount(objPtr);
  e = Tcl_GetIntFromObj(interp, objPtr, valueVar);
  Tcl_DecrRefCount(objPtr);  
  return e;
}


/* end of file */
/*
Local Variables:
mode: c
page-delimiter: "^$"
End:
*/
