# stream.test --
# 
# Part of: bztcl
# Contents: tests for the stream
# Date: Sun Feb 22, 2004
# 
# Abstract
# 
# 
# 
# Copyright (c) 2004 Marco Maggi
# 
# The author  hereby grant permission to use,  copy, modify, distribute,
# and  license this  software  and its  documentation  for any  purpose,
# provided that  existing copyright notices  are retained in  all copies
# and that  this notice  is included verbatim  in any  distributions. No
# written agreement, license, or royalty  fee is required for any of the
# authorized uses.  Modifications to this software may be copyrighted by
# their authors and need not  follow the licensing terms described here,
# provided that the new terms are clearly indicated on the first page of
# each file where they apply.
# 
# IN NO  EVENT SHALL THE AUTHOR  OR DISTRIBUTORS BE LIABLE  TO ANY PARTY
# FOR  DIRECT, INDIRECT, SPECIAL,  INCIDENTAL, OR  CONSEQUENTIAL DAMAGES
# ARISING OUT  OF THE  USE OF THIS  SOFTWARE, ITS DOCUMENTATION,  OR ANY
# DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR  HAVE BEEN  ADVISED OF  THE
# POSSIBILITY OF SUCH DAMAGE.
# 
# THE  AUTHOR  AND DISTRIBUTORS  SPECIFICALLY  DISCLAIM ANY  WARRANTIES,
# INCLUDING,   BUT   NOT  LIMITED   TO,   THE   IMPLIED  WARRANTIES   OF
# MERCHANTABILITY,    FITNESS   FOR    A    PARTICULAR   PURPOSE,    AND
# NON-INFRINGEMENT.  THIS  SOFTWARE IS PROVIDED  ON AN "AS  IS" BASIS,
# AND  THE  AUTHOR  AND  DISTRIBUTORS  HAVE  NO  OBLIGATION  TO  PROVIDE
# MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
# 

#PAGE
## ------------------------------------------------------------
## Setup.
## ------------------------------------------------------------

package require more

source	[file join [file dirname [info script]] setup.tcl]
::tcltest::configure -verbose "start pass error"
load	$libfile

#PAGE
## ---------------------------------------------------------
## Test namespace.
## ---------------------------------------------------------

namespace eval test {

    interp alias {} varchan {} more varchan
    interp alias {} pipechan {} more pipechan

    variable	ns [namespace current]
    
    # If true [debug] will print a message.
    variable	debugging 0

    proc debug { string } {
	variable	debugging
	if { !$debugging } { return }
	puts stdout [format "%-25s--- %s" test $string]
	flush stdout
    }
    
    # Variable used to enter and exit the event loop with [vwait].
    variable	forever 1

    # A counter.
    variable	counter 0
    
    # The file name.
    variable	fileName [file join [::tcltest::configure -testdir] proof.bz2]
    
    # If true the procs acting on chunks will flush the stream after
    # each operation.
    variable	flush 0
    
#PAGE
    
    variable	longData	{}
    for {set i 0} {$i < 100} {incr i} {
	append longData [expr {rand()}]
    }
    set longData [string repeat $longData 50]

    variable	shortData	[string repeat a 5000]
    variable	repetitiveData	[string repeat a 500000]
    variable	randomShortData	[string repeat [expr {rand()}] 5000]
    variable	randomLongData
    proc makeRandomLongData {} {
	variable	randomLongData
	set len 500000
#	set len 5000
	
	if {
	    (! [info exists randomLongData]) ||
	    ([string length $randomLongData] == 0)
	} {
	    for {set i 0} {$i < $len} {incr i} {
		append randomLongData [expr {rand()}]
	    }
	}
	return $randomLongData
    }

#PAGE

    proc openFileToWrite { args } {
	variable	fileName

	set channel [open $fileName {CREAT EXCL WRONLY TRUNC}]
	eval {fconfigure $channel -translation binary -encoding binary} $args
	return $channel
    }

    proc openFileToRead { args } {
	variable	fileName

	set channel [open $fileName RDONLY]
	eval {fconfigure $channel -translation binary -encoding binary} $args
	return $channel
    }

    proc writeFile { data args } {
	variable	fileName

	set channel [open $fileName {CREAT EXCL WRONLY}]
	eval {fconfigure $channel -translation binary -encoding binary} $args
	puts -nonewline $channel $data
	close $channel
    }

    proc viewFile {} {
	variable	fileName
	set channel [open $fileName RDONLY]
	fconfigure $channel -translation binary -encoding binary
	set data [read $channel]
	close $channel
	return $data
    }

    proc deleteFile {} {
	variable	fileName
	::tcltest::removeFile $fileName
    }

#PAGE

    proc read_file_single_chunk { compressOption } {
	variable	fileName
	variable	bzlib
	
	set channel [openFileToRead]
	$bzlib stream $channel RDONLY -input $compressOption
	fconfigure $channel -blocking yes
	set data [read $channel]
	close $channel
	debug "data size [string length $data]"
	return $data
    }

    proc read_file_many_chunks { compressOption } {
	variable	fileName
	variable	bzlib
	variable	flush

	set channel [openFileToRead]
	$bzlib stream $channel RDONLY -input $compressOption
	fconfigure $channel -blocking yes

	set unzipped {}
	while { ! [eof $channel] } {
	    if { $flush } {
		fconfigure $channel -flush input
	    }
	    set d [read $channel 4096]
#	    debug "read [string length $d] bytes, eof? [eof $channel]"
	    append data $d
	}

	# This [read] should  not be there because [eof]  is supposed to
	# return true only when  the transformation has signaled the EOF
	# condition;  actually  the  transformation and  its  underlying
	# channel share  the same state  strucutre: this leads  to [eof]
	# returning true when  the underlying channel is in  EOF but the
	# trasformation is not. Hence the need for another [read].

	append data [read $channel]
	close $channel
	debug "data size [string length $data]"
	return $data
    }

#PAGE

    proc write_file_single_chunk { data compressOption } {
	variable	fileName
	variable	bzlib
	
	set channel [openFileToWrite]
	$bzlib stream $channel WRONLY -output $compressOption
	fconfigure $channel -blocking yes
	puts -nonewline $channel $data
	debug "flushing"
	flush $channel
	debug "finishing"
	fconfigure $channel -finish output
	close $channel
	debug "file size [string length [viewFile]]"
    }

    proc write_file_many_chunks { data compressOption } {
	variable	fileName
	variable	bzlib
	variable	flush
	
	set channel [openFileToWrite]
	$bzlib stream $channel WRONLY -output $compressOption
	fconfigure $channel -blocking yes

	for {set i 0} {$i < [string length $data]} {incr i 4096} {
	    set chunk [string range $data $i [expr {$i+4095}]]
	    #                                          ^^^^
	    puts -nonewline $channel $chunk
	    if { $flush } {
		flush $channel
		fconfigure $channel -flush output
	    }
	}

	flush $channel
	fconfigure $channel -finish output
	close $channel
	debug "file size [string length [viewFile]]"
    }

#PAGE

proc read_event { channel } {
    variable	data
    variable	flush
    variable	forever
	

    debug "enter reader (current data length [string length $data])"
    if { [catch {
	if { ! [eof $channel] } {
	    if { $flush } {
		debug "reader: flushing input stream"
		fconfigure $channel -flush input
	    }
	    set chunk [read $channel]
	    debug "reader: read [string length $chunk] bytes"
	    append data $chunk
	} else {
	    debug "reader: finishing input"
	    fconfigure $channel -finish input
	    append data [read $channel]
	    debug "reader: closing channel $channel"
	    close $channel
	    set forever 1
	}
    } res] } {
	close $channel
	debug "reader error: $::errorInfo"
	return -code error $res
    }
    debug "exit reader (data len [string length $data])"
    return
}
proc write_event { channel original } {
    variable	counter
    variable	flush
    
    
    set delta 4095
    debug "enter writer (data len: [string length $original], counter $counter)"
    if { [catch {
	if { $counter < [string length $original] } {
	    set lastbyte [expr {$counter+$delta}]
	    set chunk [string range $original $counter $lastbyte]
	    debug [format "writing %d bytes (last byte %d)" \
		       [string length $chunk] \
		       [expr {$counter+[string length $chunk]}]]
	    puts -nonewline $channel $chunk
	    if { $flush } {
		debug "flushing output"
		flush $channel
		fconfigure $channel -flush output
	    }
	    incr counter [expr {$delta+1}]
	} else {
	    debug "writer: flushing channel"
	    flush $channel
	    debug "writer: finishing stream context"
	    fconfigure $channel -finish output
	    debug "closing channel $channel"
	    close $channel
	}
    } res] } {
	close $channel
	debug "writer error: $::errorInfo"
	return -code error $res
    }
    debug "exit writer"
    return
}

proc pipe_events { original {mode decompress} } {
    variable	ns
    variable	bzlib
    variable	counter 0
    variable	data {}
    variable	forever 0
    
    switch -exact -- $mode {
	decompress	{
	    set inputMode decompress
	    set outputMode compress
	}
	compress	{
	    deleteFile
	    write_file_single_chunk $original compress
	    set original [viewFile]
	    deleteFile
	    set inputMode compress
	    set outputMode decompress
	}
	default	{ error "bad mode: %mode" }
    }
    
    pipechan inchan ouchan RDONLY
    fconfigure $inchan -encoding binary -translation binary
    fconfigure $ouchan -encoding binary -translation binary
    
    $bzlib stream $inchan RDONLY -input $inputMode
    fconfigure $inchan -blocking no -buffering none
    
    $bzlib stream $ouchan WRONLY -output $outputMode
    fconfigure $ouchan -blocking no -buffering none
    
    fileevent $inchan readable [list ${ns}::read_event  $inchan]
    fileevent $ouchan writable [list ${ns}::write_event $ouchan $original]
    
    vwait ${ns}::forever
    debug "[string length $original]"
    debug "[string length $data]"
    string equal $original $data
    }

###
#PAGE

::tcltest::test stream-1.1 "open/close, rdonly" {
    -setup	{
	set channel	{}
    }
    -body	{
	set channel [varchan zipped RDONLY]
	$bzlib stream $channel RDONLY
	close $channel
    }
    -result	{}
    -cleanup	{
	unset -nocomplain channel zipped
    }
}

::tcltest::test stream-1.2 "open/close rdonly" {
    -setup	{
	set channel	{}
    }
    -body	{
	set channel [varchan zipped RDONLY]
	$bzlib stream $channel RDONLY -input compress
	close $channel
    }
    -result	{}
    -cleanup	{
	unset -nocomplain channel zipped
    }
}

::tcltest::test stream-1.3 "open/close" {
    -setup	{
	set channel	{}
    }
    -body	{
	set channel [varchan zipped RDONLY]
	$bzlib stream $channel RDONLY -input decompress
	close $channel
    }
    -result	{}
    -cleanup	{
	unset -nocomplain channel zipped
    }
}

#PAGE

::tcltest::test stream-2.1 "open/close, wronly" {
    -setup	{
	set channel	{}
    }
    -body	{
	set channel [varchan zipped WRONLY]
	$bzlib stream $channel WRONLY
	close $channel
    }
    -result	{}
    -cleanup	{
	unset -nocomplain channel zipped
    }
}

::tcltest::test stream-2.2 "open/close wronly" {
    -setup	{
	set channel	{}
    }
    -body	{
	set channel [varchan zipped WRONLY]
	$bzlib stream $channel WRONLY -output compress
	close $channel
    }
    -result	{}
    -cleanup	{
	unset -nocomplain channel zipped
    }
}

::tcltest::test stream-2.3 "open/close wronly" {
    -setup	{
	set channel	{}
    }
    -body	{
	set channel [varchan zipped WRONLY]
	$bzlib stream $channel WRONLY -output decompress
	close $channel
    }
    -result	{}
    -cleanup	{
	unset -nocomplain channel zipped
    }
}

#PAGE

::tcltest::test stream-3.1 "open/close, rdwr" {
    -setup	{
	set channel	{}
    }
    -body	{
	set channel [varchan zipped RDWR]
	$bzlib stream $channel RDWR
	close $channel
    }
    -result	{}
    -cleanup	{
	unset -nocomplain channel zipped
    }
}

::tcltest::test stream-3.2 "open/close rdwr" {
    -setup	{
	set channel	{}
    }
    -body	{
	set channel [varchan zipped RDWR]
	$bzlib stream $channel RDWR -input compress -output decompress
	close $channel
    }
    -result	{}
    -cleanup	{
	unset -nocomplain channel zipped
    }
}

::tcltest::test stream-3.3 "open/close rdwr" {
    -setup	{
	set channel	{}
    }
    -body	{
	set channel [varchan zipped RDWR]
	$bzlib stream $channel RDWR -input decompress -output compress
	close $channel
    }
    -result	{}
    -cleanup	{
	unset -nocomplain channel zipped
    }
}

::tcltest::test stream-3.4 "open/close rdwr" {
    -setup	{
	set channel	{}
    }
    -body	{
	set channel [varchan zipped RDWR]
	$bzlib stream $channel RDWR -input compress -output compress
	close $channel
    }
    -result	{}
    -cleanup	{
	unset -nocomplain channel zipped
    }
}

::tcltest::test stream-3.5 "open/close rdwr" {
    -setup	{
	set channel	{}
    }
    -body	{
	set channel [varchan zipped RDWR]
	$bzlib stream $channel RDWR -input decompress -output decompress
	close $channel
    }
    -result	{}
    -cleanup	{
	unset -nocomplain channel zipped
    }
}

#PAGE

::tcltest::test stream-4.1 "compress write/decompress read, blocking" {
    -setup	{
	set original	$randomShortData
	deleteFile
    }
    -body	{
	write_file_single_chunk $original compress
	string equal $original [read_file_single_chunk decompress]
    }
    -result	1
    -cleanup	{
	unset -nocomplain original
	deleteFile
    }
}

::tcltest::test stream-4.2 "compress write/decompress read, blocking" {
    -setup	{
	set original	$longData
	deleteFile
    }
    -body	{
	write_file_single_chunk $original compress
	debug "zipped file size [string length [viewFile]]"
	string equal $original [read_file_single_chunk decompress]
    }
    -result	1
    -cleanup	{
	unset -nocomplain original
	deleteFile
    }
}

::tcltest::test stream-4.3 "compress write/decompress read, blocking, chunks" {
    -setup	{
	set original	$longData
	deleteFile
    }
    -body	{
	write_file_many_chunks $original compress
	string equal $original [read_file_many_chunks decompress]
    }
    -result	1
    -cleanup	{
	unset -nocomplain original
	deleteFile
    }
}

::tcltest::test stream-4.4 "compress write/decompress read, blocking, chunks, long data" {
    -setup	{
	set original	[makeRandomLongData]
	deleteFile
    }
    -body	{
	write_file_many_chunks $original compress
	string equal $original [read_file_many_chunks decompress]
    }
    -result	1
    -cleanup	{
	unset -nocomplain channel original source zipped unzipped
	::tcltest::removeFile $fileName
    }
}

#PAGE

::tcltest::test stream-5.1 "compress read/decompress write, blocking" {
    -setup	{
	set original	$randomShortData
	deleteFile
    }
    -body	{
	writeFile $original
	set zipped [read_file_single_chunk compress]
	deleteFile
	write_file_single_chunk $zipped decompress
	string equal $original [viewFile]
    }
    -result	1
    -cleanup	{
	unset -nocomplain original
	deleteFile
    }
}

::tcltest::test stream-5.2 "compress read/decompress write, blocking, long data" {
    -setup	{
	set original	$longData
	deleteFile
    }
    -body	{
	writeFile $original
	set zipped [read_file_single_chunk compress]
	deleteFile
	write_file_single_chunk $zipped decompress
	string equal $original [viewFile]
    }
    -result	1
    -cleanup	{
	unset -nocomplain original
	deleteFile
    }
}

::tcltest::test stream-5.3 "compress read/decompress write, blocking, chunks" {
    -setup	{
	set original	$randomShortData
	deleteFile
    }
    -body	{
	writeFile $original
	set zipped [read_file_many_chunks compress]
	deleteFile
	write_file_many_chunks $zipped decompress
	string equal $original [viewFile]
    }
    -result	1
    -cleanup	{
	unset -nocomplain original
	deleteFile
    }
}

::tcltest::test stream-5.4 "compress read/decompress write, blocking, many chunks" {
    -setup	{
	set original	[makeRandomLongData]
	deleteFile
    }
    -body	{
	writeFile $original
	set zipped [read_file_many_chunks compress]
	deleteFile
	write_file_many_chunks $zipped decompress
	string equal $original [viewFile]
    }
    -result	1
    -cleanup	{
	unset -nocomplain original
	deleteFile
    }
}

#PAGE

::tcltest::test stream-6.1 "compress write/decompress read, blocking, chunks, flush" {
    -setup	{
	set original	$longData
	deleteFile
	set flush 1
    }
    -body	{
	write_file_many_chunks $original compress
	debug "zipped file size [string length [viewFile]]"
	string equal $original [read_file_many_chunks decompress]
    }
    -result	1
    -cleanup	{
	unset -nocomplain original
	deleteFile
	set flush 0
    }
}

::tcltest::test stream-6.2 "compress write/decompress read, blocking, chunks, long data, flush" {
    -setup	{
	set original	[makeRandomLongData]
	deleteFile
	set flush 1
    }
    -body	{
	write_file_many_chunks $original compress
	string equal $original [read_file_many_chunks decompress]
    }
    -result	1
    -cleanup	{
	unset -nocomplain channel original source zipped unzipped
	::tcltest::removeFile $fileName
	set flush 0
    }
}

# Remark: remember that when compressing with flush turned on, the
# result is different from the one with flush turned off; that means
# that compress+many chunks+flush cannot be compared to
# compress+single chunk.

::tcltest::test stream-6.3 "decompress write/compress read, blocking, chunks, flush" {
    -setup	{
	set original	$longData
	deleteFile
	set flush 1
    }
    -body	{
	write_file_single_chunk $original compress	
	set zipped [viewFile]
	deleteFile
	write_file_many_chunks $zipped decompress
	set compressed [read_file_many_chunks compress]
	deleteFile
	write_file_single_chunk $compressed decompress
	string equal $original [viewFile]
    }
    -result	1
    -cleanup	{
	unset -nocomplain original zipped compressed decompressed
	deleteFile
	set flush 0
    }
}

::tcltest::test stream-6.4 "decompress write/compress read, blocking, chunks, long data, flush" {
    -setup	{
	set original	[makeRandomLongData]
	deleteFile
	set flush 1
    }
    -body	{
	write_file_single_chunk $original compress	
	set zipped [viewFile]
	deleteFile
	write_file_many_chunks $zipped decompress
	set compressed [read_file_many_chunks compress]
	deleteFile
	write_file_single_chunk $compressed decompress
	string equal $original [viewFile]
    }
    -result	1
    -cleanup	{
	unset -nocomplain channel original source zipped unzipped compressed
	::tcltest::removeFile $fileName
	set flush 0
    }
}

#PAGE

::tcltest::test stream-7.1 "compress write/decompress read, events" {
    -setup	{
	set original	$randomShortData
	set flush	0
    }
    -body	{ pipe_events $original }
    -result	1
    -cleanup	{ unset -nocomplain original data }
}

::tcltest::test stream-7.2 "compress write/decompress read, events" {
    -setup	{
	set original	$longData
	set flush	0
    }
    -body	{ pipe_events $original }
    -result	1
    -cleanup	{ unset -nocomplain original data }
}

::tcltest::test stream-7.3 "compress write/decompress read, events" {
    -setup	{
	set original	[makeRandomLongData]
	set flush	0
    }
    -body	{ pipe_events $original }
    -result	1
    -cleanup	{ unset -nocomplain original data }
}

::tcltest::test stream-7.4 "decompress write/compress read, events" {
    -setup	{
	set original	$randomShortData
	set flush	0
    }
    -body	{ pipe_events $original compress }
    -result	1
    -cleanup	{ unset -nocomplain original data }
}

::tcltest::test stream-7.5 "decompress write/compress read, events" {
    -setup	{
	set original	$longData
	set flush	0
    }
    -body	{ pipe_events $original compress }
    -result	1
    -cleanup	{ unset -nocomplain original data }
}

::tcltest::test stream-7.6 "decompress write/compress read, events" {
    -setup	{
	set original	[makeRandomLongData]
	set flush	0
    }
    -body	{ pipe_events $original }
    -result	1
    -cleanup	{ unset -nocomplain original data }
}

#PAGE

::tcltest::test stream-8.1 \
    "decompress end of input stream with more data, ignore extra data" {
    -setup	{
	set original	$randomShortData
	set otherData	[string repeat a 10]
	set flush	0
	deleteFile
    }
    -body	{
	write_file_single_chunk $original compress

 	set channel [open $fileName {WRONLY APPEND}]
 	puts -nonewline $channel $otherData
 	close $channel
	debug "augmented size: [string length [viewFile]]"

	debug "reading augmented file"
	set decompressed [read_file_single_chunk decompress]
	string equal $original $decompressed
    }
    -result	1
    -cleanup	{
	unset -nocomplain original channel data other
	deleteFile
    }
}

::tcltest::test stream-8.2 \
    "decompress end of output stream with more data, ignore extra data" {
    -setup	{
	set original	$randomShortData
	set otherData	[string repeat a 10]
	set flush	0
	deleteFile
    }
    -body	{
	write_file_single_chunk $original compress

	set data [viewFile]
	deleteFile

	append data $otherData
	debug "augmented size: [string length $data]"

	set channel [openFileToWrite]
	$bzlib stream $channel WRONLY -output decompress
	fconfigure $channel -blocking yes
	puts -nonewline $channel $data
	flush $channel
	fconfigure $channel -finish output
	close $channel
	string equal $original [viewFile]
    }
    -result	1
    -cleanup	{
	unset -nocomplain original channel data other otherData
	deleteFile
    }
}

#PAGE

::tcltest::test stream-10.1 "option output buffer size" {
    -body	{
	set channel [varchan target WRONLY]
	$bzlib stream $channel WRONLY -output compress
	# this is required to force the allocation of the output buffer
	fconfigure $channel -finish output
	# here the buffer is allocated but empty, so it will be
	# reallocated to the new size
	fconfigure $channel -outputBufferSize 10
	set result [fconfigure $channel -outputBufferSize]
	close $channel
	set result
    }
    -result	10
    -cleanup	{ unset -nocomplain target result channel }
}

::tcltest::test stream-10.2 "option output buffer delta" {
    -body	{
	set channel [varchan target WRONLY]
	$bzlib stream $channel WRONLY -output compress
	fconfigure $channel -outputBufferDelta 5
	set result [fconfigure $channel -outputBufferDelta]
	close $channel
	set result
    }
    -result	5
    -cleanup	{ unset -nocomplain target result channel }
}

::tcltest::test stream-10.3 "output buffer options" {
    -body	{
	set channel [varchan target WRONLY]
	$bzlib stream $channel WRONLY -output compress
	fconfigure $channel -finish output
	fconfigure $channel -outputBufferSize 10 -outputBufferDelta 5
	set result [fconfigure $channel]
	close $channel
	set result
    }
    -result	{* -inputBufferSize 0 -outputBufferSize 10 -inputBufferDelta 0 -outputBufferDelta 5 *}
    -match	glob
    -cleanup	{ unset -nocomplain target result channel }
}

::tcltest::test stream-10.4 "option input buffer size" {
    -setup	{
	set original $shortData
	deleteFile
    }
    -body	{
	write_file_single_chunk $original compress
	
	set channel [openFileToRead]
	$bzlib stream $channel RDONLY -input decompress
	read $channel
	fconfigure $channel -inputBufferSize 99999
	set result [fconfigure $channel -inputBufferSize]
	close $channel
	set result
    }
    -result	99999
    -cleanup	{
	unset -nocomplain target result channel
	deleteFile
    }
}

::tcltest::test stream-10.5 "option input buffer delta" {
    -body	{
	set channel [varchan target RDONLY]
	$bzlib stream $channel RDONLY -input compress
	fconfigure $channel -inputBufferDelta 5
	set result [fconfigure $channel -inputBufferDelta]
	close $channel
	set result
    }
    -result	5
    -cleanup	{ unset -nocomplain target result channel }
}

::tcltest::test stream-10.6 "option input buffer" {
    -setup	{
	set original $shortData
	deleteFile
    }
    -body	{
	write_file_single_chunk $original compress
	
	set channel [openFileToRead]
	$bzlib stream $channel RDONLY -input decompress
	read $channel

	fconfigure $channel -inputBufferSize 99999 -inputBufferDelta 99
	set result [fconfigure $channel]
	close $channel
	set result
    }
    -result	{* -inputBufferSize 99999 -outputBufferSize 0 -inputBufferDelta 99 -outputBufferDelta 0}
    -match	glob
    -cleanup	{
	unset -nocomplain target result channel
	deleteFile
    }
}

#PAGE

::tcltest::test stream-100.1 "bad flush option value" {
    -body	{
	set channel [varchan target RDONLY]
	$bzlib stream $channel RDONLY -input compress
	catch {fconfigure $channel -flush woah} result
	close $channel
	set result
    }
    -result	"bad option value"
    -cleanup	{ unset -nocomplain target result channel }
}

::tcltest::test stream-100.2 "bad finish option value" {
    -body	{
	set channel [varchan target RDONLY]
	$bzlib stream $channel RDONLY -input compress
	catch {fconfigure $channel -finish woah} result
	close $channel
	set result
    }
    -result	"bad option value"
    -cleanup	{ unset -nocomplain target result channel }
}

#PAGE
## ---------------------------------------------------------
## End of "more::test" namespace.
## ---------------------------------------------------------

::tcltest::cleanupTests
}
namespace delete test


### end of file
# Local Variables:
# mode: tcl
# page-delimiter: "^#PAGE"
# End:
